<?php

namespace FM\Controller;

use FM\Controller\FMCrud;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use FM\Service\Fm00020000SolicitacoesService;
use FM\Traits\TResponse;

class Fm00020000SolicitacoesController extends FMCrud
{

    use TResponse;

    protected $dataSourceName = 'fcm_checklistagend';
    protected $SolicitacaoService;

    public function __construct($crudService, $environment, Fm00020000SolicitacoesService $SolicitacaoService)
    {
        parent::__construct($crudService, $environment);
        $this->SolicitacaoService = $SolicitacaoService;
    }

    // Busca os tipos de requisição
    public function getRequests(Request $request)
    {
        try {
            $data = $this->SolicitacaoService->getRequests($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    // Busca os tipos de requisição
    public function getRequestTypes(DTO\Response $response)
    {
        try {
            $var = $this->SolicitacaoService->getRequestTypes();
            $dtsVal = new DataSet('info', $var);
            $response->addDataSet($dtsVal);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }


    // Busca os tipos de requisição para o grid
    public function getTipoSolicitacao()
    {
        try {
            $data = $this->SolicitacaoService->getTipoSolicitacao();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    // Busca as informações de uma solicitação específica
    public function getRequestInfo(DTO\Request $request, DTO\Response $response)
    {
        try {
            $params = $request->getParameters();
            $var = $this->SolicitacaoService->getRequestInfo($params['nrsolicitacao']);
            $dtsVal = new DataSet('info', $var);
            $response->addDataSet($dtsVal);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    // Busca o histórico de uma solicitação
    public function getRequestHistoric(Request $request)
    {
        try {
            $data = $this->SolicitacaoService->getRequestHistoric($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }


    // Insere um novo tipo de requisição
    public function insertRequestType(Request $request)
    {
        try {
            $params = $request->getParameters();
            $this->SolicitacaoService->insertRequestType($params);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    // Desativa uma ou mais Tipos de solicitações
    public function inativarTipoSolicitacao(Request $request)
    {
        try {
            $params = $request->getParameters();

            $this->SolicitacaoService->inativarTipoSolicitacao($params);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    // Faz o processo de classificação da solicitação
    public function classifyRequest(Request $request)
    {
        try {
            $this->SolicitacaoService->classifyRequest($request->getParameters());
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    // Edita o tipo de solicitação
    // public function updateRequestType(DTO\Request $request)
    // {
    //     try {
    //         $params = $request->getParameters();
    //         $this->SolicitacaoService->updateRequestType($params['params']);
    //     } catch (\Exception $ex) {
    //         $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
    //     }
    // }

    public function getLocalRequest(DTO\Request $request, DTO\Response $response)
    {
        try {
            $params = $request->getParameters();
            $var = $this->SolicitacaoService->getLocalRequest($params);
            $dtsVal = new DataSet('local', $var);
            $response->addDataSet($dtsVal);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function setNewRequest(DTO\Request $request, DTO\Response $response)
    {
        try {
            $params = $request->getParameters();
            $var = $this->SolicitacaoService->setNewRequest($params);
            $dtsVal = new DataSet('request', [$var]);
            $response->addDataSet($dtsVal);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getCardsRequests(DTO\Request $request, DTO\Response $response)
    {
        try {
            $params = $request->getParameters();
            $var = $this->SolicitacaoService->getCardsRequests($params['filter'], $params['usersIds']);
            $dtsVal = new DataSet('cardsRequest', $var);
            $response->addDataSet($dtsVal);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function showRequest(DTO\Request $request, DTO\Response $response)
    {
        try {
            $params = $request->getParameters();
            $var = $this->SolicitacaoService->showRequest($params);
            $dtsVal = new DataSet('show', $var);
            $response->addDataSet($dtsVal);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getSituationRequest(DTO\Response $response)
    {
        try {
            $var = $this->SolicitacaoService->getSituationRequest();
            $dtsVal = new DataSet('situation', $var);
            $response->addDataSet($dtsVal);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getNameRequest(DTO\Response $response)
    {
        try {
            $var = $this->SolicitacaoService->getNameRequest();
            $dtsVal = new DataSet('nameRequest', $var);
            $response->addDataSet($dtsVal);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function evaluateRequest(DTO\Request $request, DTO\Response $response)
    {
        try {
            $params = $request->getParameters();
            $var = $this->SolicitacaoService->evaluateRequest($params);
            $dtsVal = new DataSet('evaluate', [$var]);
            $response->addDataSet($dtsVal);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function encerrarSolicitacao(Request $request)
    {
        try {
            $parametros = $request->getParameters();
            $this->SolicitacaoService->encerrarSolicitacao($parametros);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }
}
