<?php

namespace FM\Controller;

use Zeedhi\Framework\DTO\Request;
use FM\Service\FmtAtivosInsumosService;
use FM\Traits\TResponse;

class FmtAtivosInsumosController
{
    use TResponse;

    protected $AtivosInsumosService;

    public function __construct(FmtAtivosInsumosService $AtivosInsumosService)
    {
        $this->AtivosInsumosService = $AtivosInsumosService;
    }

    /*
     * TAB ATIVOS
     */

    public function getAtivos(Request $request)
    {
        try {
            $data = $this->AtivosInsumosService->getAtivos();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function salvarAtivo(Request $request)
    {
        try {
            $parametro = $request->getParameters();
            $this->AtivosInsumosService->salvarAtivo($parametro);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function deleteAtivos(Request $request)
    {
        try {
            $parametro = $request->getParameters();
            $this->AtivosInsumosService->deleteAtivos($parametro);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function insertFotosAtivo(Request $request)
    {
        try {
            $parametro = $request->getParameters();
            $this->AtivosInsumosService->insertFotosAtivo($parametro);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getFotosAtivo(Request $request)
    {
        try {
            $parametro = $request->getParameters();
            $data = $this->AtivosInsumosService->getFotosAtivo($parametro);
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function deleteFotoAtivo(Request $request)
    {
        try {
            $parametro = $request->getParameters();
            $data = $this->AtivosInsumosService->deleteFotoAtivo($parametro);
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getCategEquip(Request $request)
    {
        try {
            $data = $this->AtivosInsumosService->getCategEquip();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getEscopos(Request $request)
    {
        try {
            $data = $this->AtivosInsumosService->getEscopos();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    /*
     * TAB CATEGORIAS
     */

    public function getCategorias(Request $request)
    {
        try {
            $data = $this->AtivosInsumosService->getCategorias();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function salvarCategoria(Request $request)
    {
        try {
            $parametro = $request->getParameters();
            $this->AtivosInsumosService->salvarCategoria($parametro);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function deleteCategorias(Request $request)
    {
        try {
            $parametro = $request->getParameters();
            $this->AtivosInsumosService->deleteCategorias($parametro);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getCategManutencao(Request $request)
    {
        try {
            $data = $this->AtivosInsumosService->getCategManutencao();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getEquip(Request $request)
    {
        try {
            $parametros = $request->getParameters();
            $data = $this->AtivosInsumosService->getEquip($parametros);
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function editarAgrupamentos(Request $request){
        try {
            $params = $request->getParameters();
            $this->AtivosInsumosService->editarAgrupamentos($params);
           // $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    /*
     * TAB INSUMOS
     */

    public function getInsumos(Request $request)
    {
        try {
            $data = $this->AtivosInsumosService->getInsumos();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function salvarInsumo(Request $request)
    {
        try {
            $parametro = $request->getParameters();
            $this->AtivosInsumosService->salvarInsumo($parametro);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function deleteInsumos(Request $request)
    {
        try {
            $parametro = $request->getParameters();
            $this->AtivosInsumosService->deleteInsumos($parametro);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function insertFotosInsumo(Request $request)
    {
        try {
            $parametro = $request->getParameters();
            $this->AtivosInsumosService->insertFotosInsumo($parametro);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getFotosInsumo(Request $request)
    {
        try {
            $parametro = $request->getParameters();
            $data = $this->AtivosInsumosService->getFotosInsumo($parametro);
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function deleteFotoInsumo(Request $request)
    {
        try {
            $parametro = $request->getParameters();
            $data = $this->AtivosInsumosService->deleteFotoInsumo($parametro);
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getUniMedida(Request $request)
    {
        try {
            $data = $this->AtivosInsumosService->getUniMedida();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }
}
