<?php

namespace FM\Factory;

use Doctrine\ORM\EntityManager;
use Teknisa\Model\Traits\TQuery;

class FmModeloPopFactory
{
    use TQuery;

    protected $connection;
    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }
    public function getModeloPop()
    {
        $sql = "SELECT MP.NRMODELOPOP, MP.NMMODELOPOP, MP.DESCMODELOPOP, MP.LINKMODELOPOP, MP.IDATIVO, MP.NRORG,
                       AN.NRANEXOMODELOPOP, AN.NMARQUIVO, AN.DSCAMINHOARQ, AN.DSTIPO, AN.VRTAMANHO, AN.IDATIVO AS IDATIVO_ANEXO
                FROM FCM_MODELOPOP MP
                LEFT JOIN FCM_ANEXOMODELOPOP AN ON MP.NRMODELOPOP = AN.NRMODELOPOP";
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function inserirModeloPop(array $params)
    {
        $sql = "INSERT INTO FCM_MODELOPOP (
            NRMODELOPOP, NMMODELOPOP, DESCMODELOPOP, LINKMODELOPOP, IDATIVO, NRORG, DTINCLUSAO, CDOPERINCLUSAO, NRORGINCLUSAO
        ) VALUES (
            :NRMODELOPOP, :NMMODELOPOP, :DESCMODELOPOP, :LINKMODELOPOP, :IDATIVO, :NRORG, SYSDATE, :CDOPERINCLUSAO, :NRORGINCLUSAO
        )";
        
        $this->connection->executeStatement($sql, $params);
    }
    
    public function inserirAnexoModeloPop(array $params){

         $sql = "INSERT INTO FCM_ANEXOMODELOPOP (
        NRANEXOMODELOPOP, NRMODELOPOP, NMARQUIVO, DSCAMINHOARQ, DSTIPO, VRTAMANHO, IDATIVO, NRORG, DTINCLUSAO, CDOPERINCLUSAO, NRORGINCLUSAO
         ) VALUES (
        :NRANEXOMODELOPOP, :NRMODELOPOP, :NMARQUIVO, :DSCAMINHOARQ, :DSTIPO, :VRTAMANHO, :IDATIVO, :NRORG, SYSDATE, :CDOPERINCLUSAO, :NRORGINCLUSAO
         )";
    
    $this->connection->executeStatement($sql, $params);}

    public function atualizarModeloPop($params)
    {
        $sql = "UPDATE FCM_MODELOPOP
            SET NMMODELOPOP   = :NMMODELOPOP,
                CDOPERULTATU  = :CDOPERULTATU,
                DESCMODELOPOP = :DESCMODELOPOP,
                LINKMODELOPOP = :LINKMODELOPOP,
                NRORGULTATU   = :NRORGULTATU,
                DTULTATU      = SYSDATE
            WHERE NRMODELOPOP = :NRMODELOPOP";
        
        $this->connection->executeStatement($sql, $params);
    }

    public function atualizarAnexoModeloPop($params){
    $sql = "UPDATE FCM_ANEXOMODELOPOP
    SET NMARQUIVO      = :NMARQUIVO,
        DSCAMINHOARQ   = :DSCAMINHOARQ,
        DSTIPO         = :DSTIPO,
        VRTAMANHO      = :VRTAMANHO,
        CDOPERULTATU   = :CDOPERULTATU,
        NRORGULTATU    = :NRORGULTATU,
        DTULTATU       = SYSDATE
    WHERE NRANEXOMODELOPOP = :NRANEXOMODELOPOP";

    $this->connection->executeStatement($sql, $params);}

    public function excluirModeloPop($parametro)
    {
        $sql = "DELETE FROM FCM_MODELOPOP
                          WHERE NRMODELOPOP = :NRMODELOPOP";;
        $this->connection->executeStatement($sql, $parametro);
    }

    public function deletarAnexoPop($params)
    {
        $sql = "DELETE FROM FCM_ANEXOMODELOPOP
                WHERE NRANEXOMODELOPOP = :NRANEXOMODELOPOP";
        $this->connection->executeStatement($sql, $params);
    }

}