<?php

namespace FM\Service;

use FM\Factory\Fm0000000EquipesFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;

class Fm0000000EquipesService
{

    protected $EquipesFactory;
    protected $Environment;
    protected $newCode;
    protected $connection;

    public function __construct(Fm0000000EquipesFactory $Fm0000000EquipesFactory, Environment $Environment, NewCode $newCode, EntityManager $entityManager)
    {
        $this->EquipesFactory                 = $Fm0000000EquipesFactory;
        $this->Environment                    = $Environment;
        $this->newCode                        = $newCode;
        $this->connection                     = $entityManager->getConnection();
    }
    // FUNÇÕES DO FRAME PRINCIPAL EQUIPES
    public function getEquipes()
    {
        return $this->EquipesFactory->getEquipes();
    }

    public function salvarEquipes($params)
    {
        $this->connection->beginTransaction();
        try {
            $nrorg =  $this->Environment->getNrOrg();
            $operador = $this->Environment->getCdOperador();
            foreach ($params as $row) {
                if ($row["NREQUIPE"] == 0) {
                    $param = array(
                        "NREQUIPE"      => $this->newCode->getCodeByTableName("FCM_EQUIPE", $nrorg, 1, 10, true),
                        "NMEQUIPE"      => $row["NMEQUIPE"],
                        "IDATIVO"           => $row["IDATIVO"],
                        "NRORG"             => $nrorg,
                        "NRORGINCLUSAO"     => $nrorg,
                        "CDOPERINCLUSAO"    => $operador
                    );

                    $this->EquipesFactory->inserirEquipe($param);
                } else {
                    $param = array(
                        "NREQUIPE"      => $row["NREQUIPE"],
                        "NMEQUIPE"      => $row["NMEQUIPE"],
                        "IDATIVO"           => $row["IDATIVO"],
                        "CDOPERULTATU"      => $operador,
                        "NRORGULTATU"       => $nrorg
                    );

                    $this->EquipesFactory->atualizarEquipe($param);
                }
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getOperadorEquipe($params)
    {
        $param = array(
            "NREQUIPE"    => $params["EQUIPE"],
        );
        return $this->EquipesFactory->getOperadorEquipe($param);
    }

    public function relacionarOperadorEquipe($params)
    {
        $this->connection->beginTransaction();
        try {
            $nrorg =  $this->Environment->getNrOrg();
            $operador = $this->Environment->getCdOperador();

            $param = array(
                "NRRELACEQUIPEOPER"    => $this->newCode->getCodeByTableName("FCM_RELACEQUIPEOPER", $nrorg, 1, 10, true),
                "NREQUIPE"      => $params["NREQUIPE"],
                "CDOPERADOR"        => $params["OPERADOR"],
                "NRORG"             => $nrorg,
                "NRORGINCLUSAO"     => $nrorg,
                "CDOPERINCLUSAO"    => $operador,
                "IDATIVO"           => $params["IDATIVO"],
            );

            $this->EquipesFactory->relacionarOperadorEquipe($param);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function excluirEquipes($params)
    {
        $this->connection->beginTransaction();
        try {
            $filtro = '';

            $nr = implode(',', $params);
            $filtro .= "WHERE NREQUIPE IN (" . $nr . ")";
            $this->EquipesFactory->excluirOperadorEquipe($filtro);

            $this->EquipesFactory->excluirEquipes($nr);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function excluirOperadorEquipe($params)
    {
        $this->connection->beginTransaction();
        try {
            $filtro = '';

            $nr = implode(',', $params);
            $filtro .= "WHERE NRRELACEQUIPEOPER IN (" . $nr . ")";
            $this->EquipesFactory->excluirOperadorEquipe($filtro);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
}
