<?php

namespace FM\Service;

use FM\Factory\Fm00005000VistoriasFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\Remote\FileUploader\Server;
use Zeedhi\Framework\Remote\cURLRequest;
use FM\Service\FuncionariosHierarqService;
use FM\Service\FindService;

class Fm00005000VistoriasService
{

    protected $connection;
    protected $Fm00005000VistoriasFactory;
    protected $FuncionariosHierarqService;
    protected $FindService;
    protected $Environment;
    protected $newCode;

    public function __construct(EntityManager $entityManager, Fm00005000VistoriasFactory $Fm00005000VistoriasFactory, FuncionariosHierarqService $FuncionariosHierarqService, FindService $FindService, Environment $Environment, NewCode $newCode)
    {
        $this->connection                   = $entityManager->getConnection();
        $this->Fm00005000VistoriasFactory   = $Fm00005000VistoriasFactory;
        $this->FuncionariosHierarqService   = $FuncionariosHierarqService;
        $this->FindService                  = $FindService;
        $this->Environment                  = $Environment;
        $this->newCode                      = $newCode;
    }

    // Busca as Vistorias do Grid
    public function getVistorias()
    {
        $cdoperador = $this->Environment->getCdOperador();
        $listaOperadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($cdoperador);
        return $this->Fm00005000VistoriasFactory->getVistorias($listaOperadores, $cdoperador);
    }

    // Busca as atividades das vistorias
    public function getAtividades($parametros)
    {
        return $this->Fm00005000VistoriasFactory->getAtividades(array(
            "NRGERENCIAMENTOCHK"    => $parametros['NRGERENCIAMENTOCHK'],
        ));
    }

    // Buscar informaçãos para mondar os Criterios
    public function getCriterios($parametros)
    {
        return $this->Fm00005000VistoriasFactory->getCriterios(array(
            "NRATIVIDADECHK"        => $parametros['NRATIVIDADECHK'],
            "NRGERENCIAMENTOCHK"    => $parametros['NRGERENCIAMENTOCHK'],
        ));
    }

    // Edita a vistoria
    public function updateInspection($param)
    {
        $this->connection->beginTransaction();
        try {
            $nrorg =  $this->Environment->getNrOrg();
            $operador = $this->Environment->getCdOperador();
            if (!$parametros['finalizar']) {
                foreach ($parametros["DRG"] as $parametro) {
                    $salvaCriterio = array(
                        "IDMARCADO"             => $parametro['IDMARCADO'],
                        "NRCRITERIOTARGERCHK"   => $parametro['NR'],
                        "CDOPERULTATU"          => $operador,
                        "NRORGULTATU"           => $nrorg,
                        "NRGERENCIAMENTOCHK"    => $parametro['NRGERENCIAMENTOCHK']

                    );

                    $this->Fm00005000VistoriasFactory->salvarCriterios($salvaCriterio);
                }
            } else {
                $salvaVistoria = array(
                    'DSOBSTAREFAGERCHK' => $parametros['DRG'],
                    'NRGERENCIAMENTOCHK' => $parametros['NR'],
                    "CDOPERULTATU"          => $operador,
                    "NRORGULTATU"           => $nrorg
                );

                $this->Fm00005000VistoriasFactory->salvarObservacao($salvaVistoria);

                $finalizaVistoria = array(
                    'NRGERENCIAMENTOCHK'    => $parametros['NR'],
                    "CDOPERULTATU"          => $operador,
                    "NRORGULTATU"           => $nrorg
                );

                $this->Fm00005000VistoriasFactory->finalizarVistoria($finalizaVistoria);
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function insertVistoriaRecorrente($param) {
        $this->connection->beginTransaction();
        try {
            $NRORG = $this->Environment->getNrOrg();

            foreach ($param['NRCRITERIOTAREFACHK'] as $key => $criterio) {
                unset($param['NRCRITERIOTAREFACHK'][$key]['RNUM']);
                unset($param['NRCRITERIOTAREFACHK'][$key]['TOTAL_LINES']);
            }

            $dados = $this->montarDadosTarefa($param);

            $parametros = $this->validaDiasUteis($dados);

            $paramValidado = $parametros;

            if($param['RECORRENCIA'] !== 'SR') {
                $this->insertVistoria($paramValidado);

                do {
                    if ($paramValidado['TERMINARECO'] == 'N') {
                        
                        $paramValidado = $this->calcularRecorrencia($paramValidado);

                        $this->insertVistoria($paramValidado);
                    }
                } while ($paramValidado['TERMINARECO'] == 'N');
            } else {
                $this->insertVistoria($paramValidado);
            }

            return 'OK';
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }   
    }

    public function montarDadosTarefa($parametros)
    {
        $intervaloDias = $this->geraItervalo($parametros['DATERANGE'][0], (isset($parametros['DATERANGE'][1]) ? $parametros['DATERANGE'][1] : $parametros['DATERANGE'][0]));
        $parametros['DADOSTAREFAS'] = array();
        foreach ($parametros['EXECUTOR'] as $key => $dados) {
            foreach ($intervaloDias as $dia) {
                $datas = $this->calcularDuracao($dia, $dados['HORAINICIO'], $dados['DURACAO']);
                $dados['DATAINICIAL'] = $datas[0];
                $dados['DATAFINAL'] = $datas[1];
                unset($parametros['EXECUTOR'][$key]);
                array_push($parametros['DADOSTAREFAS'], $dados);
            }
        }
        return $parametros;
    }

    public function calcularDuracao($dataInicial, $horaInicial, $duracao)
    {
        // converter a duração vinda do front para o formato esperado pelo método DateInterval
        $hora = intval(substr($duracao, 0, 2));
        $minuto = intval(substr($duracao, 2, 4));
        $totalMinutos = ($hora * 60) + $minuto;

        $horarioInicial = $dataInicial . " " . $horaInicial;

        $horarioInicial = new \DateTime($horarioInicial);
        $horarioFinal = clone ($horarioInicial);

        $horarioFinal->add(new \DateInterval('PT' . $totalMinutos . 'M'));

        return [$horarioInicial->format('m/d/Y H:i'), $horarioFinal->format('m/d/Y H:i')];
    }

    public function geraItervalo($dataInicio, $dataFim)
    {
        // Definir um intervalo de 1 dia
        $fim = new \DateTime($dataFim);
        $intervalo = new \DateInterval('P1D');
        $periodo = new \DatePeriod(new \DateTime($dataInicio), $intervalo, $fim->modify('+1 day'));

        $datas = array();
        foreach ($periodo as $data) {
            array_push($datas, $data->format('m/d/Y'));
        }
        return $datas;
    }

    public function verificarConflitoDatas($dadosX, $dadosY)
    {
        // Converter as strings de data/hora para objetos DateTime
        $inicio1 = new \DateTime($dadosX['DATAINICIAL']);
        $fim1 = new \DateTime($dadosX['DATAFINAL']);
        $inicio2 = new \DateTime($dadosY['DATAINICIAL']);
        $fim2 = new \DateTime($dadosY['DATAFINAL']);

        // Verificar se há conflito de datas
        if (($inicio1 >= $inicio2 && $inicio1 <= $fim2) ||
            ($fim1 >= $inicio2 && $fim1 <= $fim2) ||
            ($inicio1 <= $inicio2 && $fim1 >= $fim2)
        ) {
            return true;
        }
        return false;
    }

    public function calcularRecorrencia($parametros) {

        // Definir a frequência da Vistoria
        switch ($parametros['RECORRENCIA']) {
            case 'Q':
                $frequencia = new \DateInterval('P14D');
                break;
            case 'M':
                $frequencia = new \DateInterval('P1M');
                break;
            case 'S':
                $frequencia = new \DateInterval('P179D');
                break;
            case 'QP':
                $frequencia = new \DateInterval('P14D');
                break;
            case 'MP':
                $frequencia = new \DateInterval('P1M');
                break;
            case 'SP':
                $frequencia = new \DateInterval('P179D');
                break;
            case 'P':
                $frequencia = new \DateInterval('P' . $parametros['QTDDIAS'] . 'D');
                break;
        }

        // Loop para adicionar as datas das próximas Vistorias
        foreach ($parametros['DADOSTAREFAS'] as $key => $executor) {
            $dataInicio = new \DateTime($executor['DATAINICIALBKP']);
            $dataFim = new \DateTime($executor['DATAFINALBKP']);

            $dataInicio = $dataInicio->add($frequencia);
            $dataFim = $dataFim->add($frequencia);
            $executor['DATAINICIAL'] = $dataInicio->format('m/d/Y H:i');
            $executor['DATAFINAL'] = $dataFim->format('m/d/Y H:i');
            $parametros['DADOSTAREFAS'][$key] = $executor;
        }

        return $this->validaDiasUteis($parametros);
    }

    public function validaDiasUteis($parametros)
    {

        foreach ($parametros['DADOSTAREFAS'] as $key => $exec) {
            $dataInicio = new \DateTime($exec['DATAINICIAL']);
            $dataFim = new \DateTime($exec['DATAFINAL']);

            $exec['DATAINICIALBKP'] = $dataInicio->format('m/d/Y H:i');
            $exec['DATAFINALBKP'] = $dataFim->format('m/d/Y H:i');

            $diaDaSemana = $dataInicio->format('N');
            $dt = $dataInicio->format('m/d/Y');

            // Verificar se a data é um dia útil
            if ($parametros['DIASEMANA'] == '2') {
                if ($diaDaSemana == 6) { // sábado
                    $dataInicio->add(new \DateInterval('P2D')); // soma dois dias para retornar à sexta-feira
                    $dataFim->add(new \DateInterval('P2D'));
                } elseif ($diaDaSemana == 7) { // domingo
                    $dataInicio->add(new \DateInterval('P1D')); // soma um dias para retornar ao último dia útil anterior, que é sexta-feira
                    $dataFim->add(new \DateInterval('P1D'));
                }
                $exec['GERATAREFA'] =  'S';
            } elseif ($parametros['DIASEMANA'] == '3') {
                $exec['GERATAREFA'] = ($diaDaSemana == 6 || $diaDaSemana == 7) ? 'N' : 'S';
                $parametros['DADOSTAREFAS'][$key] = $exec;
            } elseif ($parametros['DIASEMANA'] == '1') {
                $exec['GERATAREFA'] = 'S';
                $parametros['DADOSTAREFAS'][$key] = $exec;
            }

            $dataTermino = (isset($parametros['DTTERMINO'])) ? $parametros['DTTERMINO'] : ((isset($parametros['DATERANGE'][1])) ? $parametros['DATERANGE'][1] : $parametros['DATERANGE'][0]);

            if (new \DateTime($dt) <= new \DateTime($dataTermino)) {
                $exec['DATAINICIAL'] = $dataInicio->format('m/d/Y H:i');
                $exec['DATAFINAL'] = $dataFim->format('m/d/Y H:i');
                $parametros['DADOSTAREFAS'][$key] = $exec;
                $parametros['TERMINARECO'] =  'N';
            } else {
                $parametros['TERMINARECO'] =  'S';
                unset($parametros['DADOSTAREFAS'][$key]);
            }
        }

        return $parametros;
    }


    public function insertVistoria($param)
    {

        if($param['TERMINARECO'] == 'S') {
            return 'ok';
        }

        if($param['OBSERVACOES'] == 2) {
            $observacoes = "S";
        } else {
            $observacoes = "N";
        }

        $this->connection->beginTransaction();
        try {
            $nrorg =  $this->Environment->getNrOrg();
            $operador = $this->Environment->getCdOperador();

            foreach($param['DADOSTAREFAS'] as $parametros) {
                $nrgerenciamentochk = $this->newCode->getCodeByTableName("FCM_GERENCIAMENTOCHK", $nrorg, 1, 10, true);
                $dataInicial = new \DateTime($parametros['DATAINICIAL']);
                $dataFinal = new \DateTime($parametros['DATAFINAL']);
                $params = array(
                    "NRGERENCIAMENTOCHK"    => $nrgerenciamentochk,
                    "NMGERENCIAMENTOCHK"    => $param['NOMETAREFA'],
                    "NRLOCALCLIENTE"        => $parametros['NRLOCALCLIENTE'],
                    "NRCHECKLIST"           => $param['NRCHECKLIST'],
                    "CDOPERADOR"            => $parametros['ENCARREGADO'],
                    "DTHORAAGEND"           => $dataInicial->format('d/m/Y H:i'),
                    "DTHRFINALAGEND"        => $dataFinal->format('d/m/Y H:i'),
                    "OPERADOR"              => $operador,
                    "NRCHECKLISTAGEND"      => NULL,
                    "IDOBRIGAOBSERVACAO"    => $observacoes,
                    "NRSEQCONTRATO"         => (isset($parametros['NRSEQCONTRATO'])) ? $parametros['NRSEQCONTRATO'] : null
                );
    
                $this->Fm00005000VistoriasFactory->insertChk($params);

                $NRGERCHKHISTORICO = $this->newCode->getCodeByTableName("FCM_GERCHKHISTORICO", $nrorg, 1, 10, true);
                $parametroHistorico = array(
                    "NRGERCHKHISTORICO"  => $NRGERCHKHISTORICO,
                    "NRGERENCIAMENTOCHK" => $nrgerenciamentochk,
                    "NMGERENCIAMENTOCHK" => $param['NOMETAREFA'],
                    "DSGERENCIAMENTOCHK" => "Vistoria criada.",
                    "IDTIPOHISTORICO"    => 1,
                    "NRORG"              => $nrorg,
                    "CDOPERINCLUSAO"     => $operador,
                    "NRORGINCLUSAO"      => $nrorg,
                    "CDOPERULTATU"       => $operador,
                    "NRORGULTATU"        => $nrorg
                );

                $this->Fm00005000VistoriasFactory->atualizaHistoricoVistoria($parametroHistorico);
    
                foreach ($param['NRTAREFACHK'] as $atividade) {
                    $nrtarefaagendchk = $this->newCode->getCodeByTableName("FCM_TAREFAGERCHK", $nrorg, 1, 10, true);
                    $paramsTarefa = array(
                        "NRTAREFAGERCHK"        => $nrtarefaagendchk,
                        "NRGERENCIAMENTOCHK"    => $nrgerenciamentochk,
                        "NRTAREFACHK"           => $atividade,
                        "CDOPERADOR"            => $parametros['ENCARREGADO'],
                        "DTHORATAREFAGERCHK"    => $dataInicial->format('d/m/Y H:i'),
                        "DSOBSTAREFAGERCHK"     => null,
                        "OPERADOR"              => $operador
                    );
    
                    $this->Fm00005000VistoriasFactory->insertChkTarefa($paramsTarefa);
    
                    foreach ($param['NRCRITERIOTAREFACHK'] as $criterios) {
                        $paramsValida = array(
                            "NRTAREFACHK"      => $atividade,
                            "NRCRITERIOTAREFA" => $criterios['NRCRITERIOTAREFA']
                        );
    
                        $valida = $this->Fm00005000VistoriasFactory->validaCriterio($paramsValida);
    
                        if ($valida > 0) {
                            $nrcriteriostaragerchk = $this->newCode->getCodeByTableName("FCM_CRITERIOTARGERCHK", $nrorg, 1, 10, true);
                            $paramsCriterio = array(
                                "NRCRITERIOTARGERCHK" => $nrcriteriostaragerchk,
                                "NMCRITERIOTARGERCHK" => $criterios['DSCRITERIOTAREFA'],
                                "NRTAREFAGERCHK"      => $nrtarefaagendchk,
                                "NRCRITERIOTAREFACHK" => ($criterios['NRCRITERIOTAREFACHK'] === $criterios['DSCRITERIOTAREFA']) ? null : $criterios['NRCRITERIOTAREFACHK'],
                                "OPERADOR"            => $operador
                            );
    
                            $this->Fm00005000VistoriasFactory->insertChkCriterio($paramsCriterio);
                        }
                    }
                }
            };
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteVistorias($param) 
    {

        // ainda falta criar validaçao e deletar fotos
        $this->connection->beginTransaction();
        try {
            foreach($param['ROWS'] as $row) {

                $nrtarefagerchk = $this->Fm00005000VistoriasFactory->getChkTarefa($row);
                $nrgerenciamentochk = $row;
                $this->Fm00005000VistoriasFactory->deleteImagensVistoria($nrgerenciamentochk);
                foreach($nrtarefagerchk['data'] as $nr) {
                    $this->Fm00005000VistoriasFactory->deleteCriterios($nr);
                    $this->Fm00005000VistoriasFactory->deleteChkTarefa($nr);
                }
                $this->Fm00005000VistoriasFactory->deleteGerenciamentoChk($nrgerenciamentochk);
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    
    public function salvarCriterios($parametros) {
    
        $this->connection->beginTransaction();
        $NRORG = $this->Environment->getNrOrg();
        $cdOperador = $this->Environment->getCdOperador();

        try {
            foreach($parametros["ARRAYOBS"] as $param) {
                $this->Fm00005000VistoriasFactory->updateCriterios(array(
                    "NRATIVIDADECHK"     => $param['NRATIVIDADECHK'],
                    "NRGERENCIAMENTOCHK" => $param['NRGERENCIAMENTOCHK'],
                    "IDMARCADO"          => "N",
                    "CDOPERULTATU"       => $cdOperador,
                    "NRORGULTATU"        => $NRORG,
                ), "");

                if (!empty($param['CRITERIOS'])) {
                    $criterios = $this->FindService->formatarArray($param['CRITERIOS']);
    
                    $paramQuery = " AND NMCRITERIOTARGERCHK IN(" . $criterios . ")";
    
                    $this->Fm00005000VistoriasFactory->updateCriterios(array(
                        "NRATIVIDADECHK"     => $param['NRATIVIDADECHK'],
                        "NRGERENCIAMENTOCHK" => $param['NRGERENCIAMENTOCHK'],
                        "IDMARCADO"          => 'S',
                        "CDOPERULTATU"       => $cdOperador,
                        "NRORGULTATU"        => $NRORG,
                    ), $paramQuery);
                }
            }

            $this->Fm00005000VistoriasFactory->updateStatusVistoria(array(
                "IDSITUACAOGERCHK"   => '2',
                "NRGERENCIAMENTOCHK" => $parametros['NRGERENCIAMENTOCHK'],
                "CDOPERULTATU"       => $cdOperador,
                "NRORGULTATU"        => $NRORG,
            ));
            
            $NRGERCHKHISTORICO = $this->newCode->getCodeByTableName("FCM_GERCHKHISTORICO", $NRORG, 1, 10, true);
            $parametroHistorico = array(
                "NRGERCHKHISTORICO"  => $NRGERCHKHISTORICO,
                "NRGERENCIAMENTOCHK" => $parametros['NRGERENCIAMENTOCHK'],
                "NMGERENCIAMENTOCHK" => $parametros['NMGERENCIAMENTOCHK'],
                "DSGERENCIAMENTOCHK" => "Vistoria em andamento.",
                "IDTIPOHISTORICO"    => 3,
                "NRORG"              => $NRORG,
                "CDOPERINCLUSAO"     => $cdOperador,
                "NRORGINCLUSAO"      => $NRORG,
                "CDOPERULTATU"       => $cdOperador,
                "NRORGULTATU"        => $NRORG
            );

            $this->Fm00005000VistoriasFactory->atualizaHistoricoVistoria($parametroHistorico);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function salvarObservacao($parametros)
    {
        try{
            $this->connection->beginTransaction();
            $NRORG = $this->Environment->getNrOrg();
            $cdOperador = $this->Environment->getCdOperador();
    
            $this->Fm00005000VistoriasFactory->salvarObservacao(array(
                'DSOBSTAREFAGERCHK' => $parametros['DSOBSTAREFAGERCHK'],
                'NRGERENCIAMENTOCHK' => $parametros['NRGERENCIAMENTOCHK'],
                "CDOPERULTATU"          => $cdOperador,
                "NRORGULTATU"           => $NRORG,
            ));
            
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getResponsaveisVistoria($parametros)
    {
        if (!empty($parametros['NRLOCALCLIENTE'])) {
            $locais = is_array($parametros['NRLOCALCLIENTE']) ? $this->FindService->formatarArray($parametros['NRLOCALCLIENTE']) : "'" . $parametros['NRLOCALCLIENTE'] . "'";
            return $this->Fm00005000VistoriasFactory->getResponsaveisVistoria($locais);
        }
    }

    public function finalizarVistoria($parametros)
    {
        $this->connection->beginTransaction();
        $NRORG = $this->Environment->getNrOrg();
        $cdOperador = $this->Environment->getCdOperador();

        try {
            foreach ($parametros['ATIVIDADES'] as $atividade) {
                $this->Fm00005000VistoriasFactory->updateObservacao(array(
                    "NRATIVIDADECHK"     => $atividade,
                    "NRGERENCIAMENTOCHK" => $parametros['NRGERENCIAMENTOCHK'],
                    "DSOBSTAREFAGERCHK"  => $parametros['DSOBSTAREFAGERCHK'],
                    "CDOPERULTATU"       => $cdOperador,
                    "NRORGULTATU"        => $NRORG,
                ));
            }
            
            $this->Fm00005000VistoriasFactory->updateStatusVistoria(array(
                "IDSITUACAOGERCHK"   => '3',
                "NRGERENCIAMENTOCHK" => $parametros['NRGERENCIAMENTOCHK'],
                "CDOPERULTATU"       => $cdOperador,
                "NRORGULTATU"        => $NRORG,
            ));

            $NRGERCHKHISTORICO = $this->newCode->getCodeByTableName("FCM_GERCHKHISTORICO", $NRORG, 1, 10, true);
            if (!$parametros["POSSUINAOCONFORM"]) {
                $parametroHistoricoVistoria = array(
                    "NRGERCHKHISTORICO"  => $NRGERCHKHISTORICO,
                    "NRGERENCIAMENTOCHK" => $parametros['NRGERENCIAMENTOCHK'],
                    "NMGERENCIAMENTOCHK" => $parametros['NMGERENCIAMENTOCHK'],
                    "DSGERENCIAMENTOCHK" => "Vistoria finalizada sem não conformidade.",
                    "IDTIPOHISTORICO"    => 4,
                    "NRORG"              => $NRORG,
                    "CDOPERINCLUSAO"     => $cdOperador,
                    "NRORGINCLUSAO"      => $NRORG,
                    "CDOPERULTATU"       => $cdOperador,
                    "NRORGULTATU"        => $NRORG
                );
    
                $this->Fm00005000VistoriasFactory->atualizaHistoricoVistoria($parametroHistoricoVistoria);

                if (isset($parametros['NRCHECKLISTAGEND'])) {
                    $NRCHKAGENDHISTORICO = $this->newCode->getCodeByTableName("FCM_CHKAGENDHISTORICO", $NRORG, 1, 10, true);
                    $parametroHistoricoTarefa = array(
                        "NRCHKAGENDHISTORICO" => $NRCHKAGENDHISTORICO,
                        "NRCHECKLISTAGEND"    => $parametros['NRCHECKLISTAGEND'],
                        "NMCHECKLISTAGEND"    => $parametros['NMGERENCIAMENTOCHK'],
                        "DSCHKAGENDHISTORICO" => "Tarefa avaliada e finalizada.",
                        "IDTIPOHISTORICO"     => 4,
                        "NRORG"               => $NRORG,
                        "CDOPERINCLUSAO"      => $cdOperador,
                        "NRORGINCLUSAO"       => $NRORG,
                        "CDOPERULTATU"        => $cdOperador,
                        "NRORGULTATU"         => $NRORG
                    );
    
                    $this->Fm00005000VistoriasFactory->atualizaHistoricoTarefa($parametroHistoricoTarefa);
                }
            } else {
                $parametroHistorico = array(
                    "NRGERCHKHISTORICO"  => $NRGERCHKHISTORICO,
                    "NRGERENCIAMENTOCHK" => $parametros['NRGERENCIAMENTOCHK'],
                    "NMGERENCIAMENTOCHK" => $parametros['NMGERENCIAMENTOCHK'],
                    "DSGERENCIAMENTOCHK" => "Vistoria finalizada contendo não conformidades.",
                    "IDTIPOHISTORICO"    => 6,
                    "NRORG"              => $NRORG,
                    "CDOPERINCLUSAO"     => $cdOperador,
                    "NRORGINCLUSAO"      => $NRORG,
                    "CDOPERULTATU"       => $cdOperador,
                    "NRORGULTATU"        => $NRORG
                );
    
                $this->Fm00005000VistoriasFactory->atualizaHistoricoVistoria($parametroHistorico);

                if (isset($parametros['NRCHECKLISTAGEND'])) {
                    $NRCHKAGENDHISTORICO = $this->newCode->getCodeByTableName("FCM_CHKAGENDHISTORICO", $NRORG, 1, 10, true);
                    $parametroHistoricoTarefa = array(
                        "NRCHKAGENDHISTORICO" => $NRCHKAGENDHISTORICO,
                        "NRCHECKLISTAGEND"    => $parametros['NRCHECKLISTAGEND'],
                        "NMCHECKLISTAGEND"    => $parametros['NMGERENCIAMENTOCHK'],
                        "DSCHKAGENDHISTORICO" => "Tarefa avaliada e finalizada contendo não conformidades.",
                        "IDTIPOHISTORICO"     => 6,
                        "NRORG"               => $NRORG,
                        "CDOPERINCLUSAO"      => $cdOperador,
                        "NRORGINCLUSAO"       => $NRORG,
                        "CDOPERULTATU"        => $cdOperador,
                        "NRORGULTATU"         => $NRORG
                    );
    
                    $this->Fm00005000VistoriasFactory->atualizaHistoricoTarefa($parametroHistoricoTarefa);
                }
            }


            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function excluirImagensVistoria($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $this->Fm00005000VistoriasFactory->excluirImagensVistoria(array(
                "NRGERENCIAMENTOCHK" => $parametros['NRGERENCIAMENTOCHK'],
                "NRFOTOTAREFAGERCHK" => $parametros['NRFOTOTAREFAGERCHK'],
            ));

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function voltarTarefa($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->Environment->getNrOrg();
            $cdOperador = $this->Environment->getCdOperador();

            $parametroTarefa = array(
                "NRCHECKLISTAGEND"  => $parametros["NRCHECKLISTAGEND"],
                "IDSITUACAOAGEND"   => 1,
                "NRORGULTATU"       => $nrorg,
                "CDOPERULTATU"      => $cdOperador
            );
            $this->Fm00005000VistoriasFactory->atualizaStatusTarefa($parametroTarefa);

            $parametroVistoria = array(
                "NRGERENCIAMENTOCHK" => $parametros["NRGERENCIAMENTOCHK"],
                "IDSITUACAOGERCHK"   => 4,
                "NRORGULTATU"        => $nrorg,
                "CDOPERULTATU"       => $cdOperador
            );
            $this->Fm00005000VistoriasFactory->atualizaStatusVistoria($parametroVistoria);

            foreach ($parametros["ATIVIDADES"] as $atividade) {
                $parametroTarefa = array(
                    "NRCHECKLISTAGEND"  => $parametros["NRCHECKLISTAGEND"],
                    "NRATIVIDADECHK"    => $atividade,
                    "NRORGULTATU"       => $nrorg,
                    "CDOPERULTATU"      => $cdOperador
                );
                $this->Fm00005000VistoriasFactory->atualizaStatusTarefaAgend($parametroTarefa);
            }

            $NRGERCHKHISTORICO = $this->newCode->getCodeByTableName("FCM_GERCHKHISTORICO", $nrorg, 1, 10, true);
            $parametroHistoricoVistoria = array(
                "NRGERCHKHISTORICO"  => $NRGERCHKHISTORICO,
                "NRGERENCIAMENTOCHK" => $parametros['NRGERENCIAMENTOCHK'],
                "NMGERENCIAMENTOCHK" => $parametros['NMGERENCIAMENTOCHK'],
                "DSGERENCIAMENTOCHK" => "Foram encontradas não conformidades. Aguardando correção da tarefa.",
                "IDTIPOHISTORICO"    => 5,
                "NRORG"              => $nrorg,
                "CDOPERINCLUSAO"     => $cdOperador,
                "NRORGINCLUSAO"      => $nrorg,
                "CDOPERULTATU"       => $cdOperador,
                "NRORGULTATU"        => $nrorg
            );

            $this->Fm00005000VistoriasFactory->atualizaHistoricoVistoria($parametroHistoricoVistoria);

            $NRCHKAGENDHISTORICO = $this->newCode->getCodeByTableName("FCM_CHKAGENDHISTORICO", $nrorg, 1, 10, true);
            $parametroHistoricoTarefa = array(
                "NRCHKAGENDHISTORICO" => $NRCHKAGENDHISTORICO,
                "NRCHECKLISTAGEND"    => $parametros['NRCHECKLISTAGEND'],
                "NMCHECKLISTAGEND"    => $parametros['NMGERENCIAMENTOCHK'],
                "DSCHKAGENDHISTORICO" => "Foram encontradas não conformidades na vistoria. Tarefa aguarda correção.",
                "IDTIPOHISTORICO"     => 5,
                "NRORG"               => $nrorg,
                "CDOPERINCLUSAO"      => $cdOperador,
                "NRORGINCLUSAO"       => $nrorg,
                "CDOPERULTATU"        => $cdOperador,
                "NRORGULTATU"         => $nrorg
            );

            $this->Fm00005000VistoriasFactory->atualizaHistoricoTarefa($parametroHistoricoTarefa);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    
    public function getHistoricoVistoria($parametros)
    {
        return $this->Fm00005000VistoriasFactory->getHistoricoVistoria(array(
            "NRGERENCIAMENTOCHK" => $parametros['NRGERENCIAMENTOCHK'],
        ));
    }
}