<?php

namespace FM\Service;

use FM\Factory\Fm00030000TarefasRecorrentesFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use FM\Service\FuncionariosHierarqService;
use FM\Service\Fm00004000TarefasService;
use FM\Service\FindService;

class Fm00030000TarefasRecorrentesService
{

    protected $Fm00030000TarefasRecorrentesFactory;
    protected $Environment;
    protected $newCode;
    protected $FuncionariosHierarqService;
    protected $Fm00004000TarefasService;
    protected $connection;
    protected $FindService;

    public function __construct(EntityManager $entityManager, Fm00030000TarefasRecorrentesFactory $Fm00030000TarefasRecorrentesFactory, FuncionariosHierarqService $FuncionariosHierarqService, Environment $Environment, NewCode $newCode, Fm00004000TarefasService $Fm00004000TarefasService, FindService $FindService)
    {
        $this->connection                           = $entityManager->getConnection();
        $this->Fm00030000TarefasRecorrentesFactory  = $Fm00030000TarefasRecorrentesFactory;
        $this->FuncionariosHierarqService           = $FuncionariosHierarqService;
        $this->Environment                          = $Environment;
        $this->newCode                              = $newCode;
        $this->Fm00004000TarefasService             = $Fm00004000TarefasService;
        $this->FindService                          = $FindService;
    }


    public function insertTarefasRecorrentes($param)
    {
        $this->connection->beginTransaction();
        try {
            $NRORG = $this->Environment->getNrOrg();
            $datasConflitantes = array();

            foreach ($param['NRCRITERIOTAREFACHK'] as $key => $criterio) {
                unset($param['NRCRITERIOTAREFACHK'][$key]['RNUM']);
                unset($param['NRCRITERIOTAREFACHK'][$key]['TOTAL_LINES']);
            }
            
            $dados = ($param['CONFLITO'] === true) ?  $this->montarDadosPosConflito($param) : $this->montarDadosTarefa($param);
            
            $parametros = $this->validaDiasUteis($dados);
            
            // Validar se há conflito entre as tarefas que estão sendo inseridas
            if ($param['PARAMETRIZACAO'] !== 'N' && !empty($parametros['DADOSTAREFAS']['FUNCIONARIO'])) {
                $paramValidado = $this->validarConflitos($parametros, true);

                if (count($paramValidado[1]) !== 0) {
                    throw new \Exception("Operação bloqueada!<br><br> Existem conflitos de horário nas tarefas que está tentando inserir. Altere os horários, ou o Operador, ou desative a parametrização para inserir mais de uma tarefa para o mesmo horário.");
                }
                unset($paramValidado[1]);
            } else {
                $paramValidado[0] = $parametros;
            }

            
            if (!empty($parametros['DADOSTAREFAS']['FUNCIONARIO'])) {
                // Validar se há conflitos com as tarefas do backend
                $paramValidado = $this->validarConflitos($paramValidado[0], false);
            }

            if (!empty($paramValidado[1]) && count($paramValidado[1]) > 0) {
                array_push($datasConflitantes, $paramValidado[1]);
            }
            unset($paramValidado[1]);

            if ($param['RECORRENCIA'] !== 'SR') {
                $nrTarefaRecorrente = $this->newCode->getCodeByTableName("FCM_TAREFARECORRENTE", $NRORG, 1, 10, true);
                $paramValidado[0]['NRTAREFARECORRENTE'] = $nrTarefaRecorrente;

                $this->inserirTarefaRecorrente($paramValidado[0]);
                $this->Fm00004000TarefasService->insertNovaTarefa($paramValidado[0]);

                do {
                    if ($paramValidado[0]['TERMINARECO'] == 'N') {
                        $paramValidado = $this->calcularRecorrenciaPeriodo($paramValidado[0]);
                        $paramValidado = $this->validarConflitos($paramValidado, false);

                        if (count($paramValidado[1]) > 0) {
                            array_push($datasConflitantes, $paramValidado[1]);
                        }
                        unset($paramValidado[1]);

                        $this->Fm00004000TarefasService->insertNovaTarefa($paramValidado[0]);
                    }
                } while ($paramValidado[0]['TERMINARECO'] == 'N');
            } else {
                $this->Fm00004000TarefasService->insertNovaTarefa($paramValidado[0]);
            }

            if (count($datasConflitantes) > 0 && $param['PARAMETRIZACAO'] !== 'N') {
                $retorno = $this->formatarRetorno($paramValidado, $datasConflitantes);
                unset($datasConflitantes);
            } else {
                $retorno = 'OK';
            }
            $this->connection->commit();
            return $retorno;
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function formatarRetorno($parametros, $datasConflitantes)
    {
        $backup = array();
        $parametros[0]['DADOSCONFLITO'] = array();

        foreach ($datasConflitantes as $mes) {
            foreach ($mes as $dia) {
                $arr = [$dia['DATAINICIAL'], $dia['FUNCIONARIO'], $dia['NRLOCALCLIENTE']];
                if (array_diff($arr, $backup)) {
                    array_push($parametros[0]['DADOSCONFLITO'], $dia);
                }
                $backup = $arr;
            }
        }
        return $parametros[0];
    }

    public function inserirTarefaRecorrente($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $NRORG = $this->Environment->getNrOrg();
            $operador = $this->Environment->getCdOperador();

            $this->Fm00030000TarefasRecorrentesFactory->insertTarefasRecorrentes(array(
                "NRTAREFARECORRENTE"    => $parametros['NRTAREFARECORRENTE'],
                "NMTAREFA"              => $parametros['NOMETAREFA'],
                "NRCHECKLIST"           => $parametros['NRCHECKLIST'],
                "ITENSCHECKLIST"        => implode(",", $parametros['NRATIVIDADECHK']),
                "ESCOPOSTAREFA"         => implode(",", $parametros['NRESCOPOSTAREFA']),
                "ITENSTAREFA"           => implode(",", $parametros['NRTAREFACHK']),
                "CRITERIOS"             => json_encode($parametros['NRCRITERIOTAREFACHK']),
                "DIASSEMANA"            => $parametros['DIASEMANA'],
                "IDCRIARVISTORIA"       => $parametros['VISTORIA'],
                "NRORG"                 => $NRORG,
                "IDATIVO"               => "S",
                "CDOPERINCLUSAO"        => $operador,
                "NRORGINCLUSAO"         => $NRORG,
                "IDSITUACAO"            => "1",
                "CDCLIENTE"             => $parametros['CDCLIENTE'],
                "NRSEQCONTRATO"         => isset($parametros['DADOSTAREFAS'][0]['NRSEQCONTRATO']) ? $parametros['DADOSTAREFAS'][0]['NRSEQCONTRATO'] : null,
                "DADOSRECORRENCIA"      => json_encode($parametros['DADOSTAREFAS']),
                "IDTIPORECORRENCIA"     => $parametros['RECORRENCIA'],
            ));
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function calcularRecorrenciaPeriodo($parametros)
    {
        // Definir a frequência da tarefa
        switch ($parametros['RECORRENCIA']) {
            case 'Q':
                $frequencia = new \DateInterval('P14D');
                break;
            case 'M':
                $frequencia = new \DateInterval('P1M');
                break;
            case 'S':
                $frequencia = new \DateInterval('P179D');
                break;
            case 'QP':
                $frequencia = new \DateInterval('P14D');
                break;
            case 'MP':
                $frequencia = new \DateInterval('P1M');
                break;
            case 'SP':
                $frequencia = new \DateInterval('P179D');
                break;
            case 'P':
                $frequencia = new \DateInterval('P' . $parametros['QTDDIAS'] . 'D');
                break;
        }

        // Loop para adicionar as datas das próximas tarefas
        foreach ($parametros['DADOSTAREFAS'] as $key => $executor) {
            $dataInicio = new \DateTime($executor['DATAINICIALBKP']);
            $dataFim = new \DateTime($executor['DATAFINALBKP']);

            $dataInicio = $dataInicio->add($frequencia);
            $dataFim = $dataFim->add($frequencia);
            $executor['DATAINICIAL'] = $dataInicio->format('m/d/Y H:i');
            $executor['DATAFINAL'] = $dataFim->format('m/d/Y H:i');
            $parametros['DADOSTAREFAS'][$key] = $executor;
        }

        return $this->validaDiasUteis($parametros);
    }

    public function geraItervalo($dataInicio, $dataFim)
    {
        // Definir um intervalo de 1 dia
        $fim = new \DateTime($dataFim);
        $intervalo = new \DateInterval('P1D');
        $periodo = new \DatePeriod(new \DateTime($dataInicio), $intervalo, $fim->modify('+1 day'));

        $datas = array();
        foreach ($periodo as $data) {
            array_push($datas, $data->format('m/d/Y'));
        }
        return $datas;
    }

    public function validaDiasUteis($parametros)
    {

        foreach ($parametros['DADOSTAREFAS'] as $key => $exec) {
            $dataInicio = new \DateTime($exec['DATAINICIAL']);
            $dataFim = new \DateTime($exec['DATAFINAL']);

            $exec['DATAINICIALBKP'] = $dataInicio->format('m/d/Y H:i');
            $exec['DATAFINALBKP'] = $dataFim->format('m/d/Y H:i');

            $diaDaSemana = $dataInicio->format('N');
            $dt = $dataInicio->format('m/d/Y');

            // Verificar se a data é um dia útil
            if ($parametros['DIASEMANA'] == '2') {
                if ($diaDaSemana == 6) { // sábado
                    $dataInicio->add(new \DateInterval('P2D')); // soma dois dias para retornar à sexta-feira
                    $dataFim->add(new \DateInterval('P2D'));
                } elseif ($diaDaSemana == 7) { // domingo
                    $dataInicio->add(new \DateInterval('P1D')); // soma um dias para retornar ao último dia útil anterior, que é sexta-feira
                    $dataFim->add(new \DateInterval('P1D'));
                }
                $exec['GERATAREFA'] =  'S';
            } elseif ($parametros['DIASEMANA'] == '3') {
                $exec['GERATAREFA'] = ($diaDaSemana == 6 || $diaDaSemana == 7) ? 'N' : 'S';
                $parametros['DADOSTAREFAS'][$key] = $exec;
            } elseif ($parametros['DIASEMANA'] == '1') {
                $exec['GERATAREFA'] = 'S';
                $parametros['DADOSTAREFAS'][$key] = $exec;
            }

            $dataTermino = (isset($parametros['DTTERMINO'])) ? $parametros['DTTERMINO'] : ((isset($parametros['DATERANGE'][1])) ? $parametros['DATERANGE'][1] : $parametros['DATERANGE'][0]);

            if (new \DateTime($dt) <= new \DateTime($dataTermino)) {
                $exec['DATAINICIAL'] = $dataInicio->format('m/d/Y H:i');
                $exec['DATAFINAL'] = $dataFim->format('m/d/Y H:i');
                $parametros['DADOSTAREFAS'][$key] = $exec;
                $parametros['TERMINARECO'] =  'N';
            } else {
                $parametros['TERMINARECO'] =  'S';
                unset($parametros['DADOSTAREFAS'][$key]);
            }
        }

        return $parametros;
    }

    public function validarConflitos($parametros, $front)
    {

        $datasConflitantes = array();
        if ($front) {
            for ($i = 0; $i < count($parametros['DADOSTAREFAS']); $i++) {
                for ($j = $i + 1; $j < count($parametros['DADOSTAREFAS']); $j++) {
                    if (
                        $parametros['DADOSTAREFAS'][$i]['FUNCIONARIO'] == $parametros['DADOSTAREFAS'][$j]['FUNCIONARIO']
                    ) {
                        if ($this->verificarConflitoDatas($parametros['DADOSTAREFAS'][$i], $parametros['DADOSTAREFAS'][$j])) {
                            $parametros['DADOSTAREFAS'][$i]['GERATAREFA'] = 'N';
                            $parametros['DADOSTAREFAS'][$j]['GERATAREFA'] = 'N';
                            $parametros['DADOSTAREFAS'][$i]['AGENDAMENTOATUAL'] = 'S';
                            $parametros['DADOSTAREFAS'][$j]['AGENDAMENTOATUAL'] = 'S';

                            array_push($datasConflitantes, $parametros['DADOSTAREFAS'][$i]);
                            array_push($datasConflitantes, $parametros['DADOSTAREFAS'][$j]);
                        }
                    }
                }
            }
        } else {
            $operadores = array();

            if (count($parametros['DADOSTAREFAS']) > 0) {
                foreach ($parametros['DADOSTAREFAS'] as $chave => $dado) {
                    array_push($operadores, $dado['FUNCIONARIO']);
                }
                $valor = $this->FindService->formatarArray(array_unique($operadores));
                $dadosBanco = $this->Fm00030000TarefasRecorrentesFactory->getTarefasPorOperador($valor);

                foreach ($parametros['DADOSTAREFAS'] as $chave => $dadoFront) {
                    foreach ($dadosBanco as $dadoBack) {
                        if ($dadoFront['FUNCIONARIO'] == $dadoBack['CDOPERADOR']) {
                            if ($this->verificarConflitoDatas($dadoFront, $dadoBack)) {
                                $dadoFront['GERATAREFA'] = ($parametros['PARAMETRIZACAO'] === 'N') ? 'S' : 'N';
                                $dadoFront['AGENDAMENTOATUAL'] = 'N';
                                $parametros['DADOSTAREFAS'][$chave] = $dadoFront;
                                array_push($datasConflitantes, $dadoFront);
                            }
                        }
                    }
                }
            }
        }
        return [$parametros, $datasConflitantes];
    }

    public function verificarConflitoEdicaoDeTarefas($parametros): array
    {
        $operadores = array();
        $datasConflitantes = array();
        if (count($parametros['DADOSTAREFAS']) > 0) {
            foreach ($parametros['DADOSTAREFAS'] as $chave => $dado) {
                array_push($operadores, $dado['FUNCIONARIO']);
            }

            $valor = $this->FindService->formatarArray(array_unique($operadores));
            $dadosBanco = $this->Fm00030000TarefasRecorrentesFactory->getTarefasPorOperador($valor);

            foreach ($parametros['DADOSTAREFAS'] as $chave => $dadoFront) {
                foreach ($dadosBanco as $dadoBack) {
                    if ($dadoFront['FUNCIONARIO'] == $dadoBack['CDOPERADOR']) {
                        if ($this->verificarConflitoDatas($dadoFront, $dadoBack)) {
                            if ($dadoFront["NRCHECKLISTAGEND"] == $dadoBack["NRCHECKLISTAGEND"]) {
                                $dadoFront['GERATAREFA'] = ($parametros['PARAMETRIZACAO'] === 'N') ? 'S' : 'N';
                                $dadoFront['AGENDAMENTOATUAL'] = 'S';
                                $parametros['DADOSTAREFAS'][$chave] = $dadoFront;
                                return ['OK'];
                            } else {
                                $datasConflitantes = $dadoBack;
                                return $datasConflitantes;
                            }
                        }
                    }
                }
            }
        }
        return ['OK'];
    }

    public function verificarConflitoDatas($dadosX, $dadosY)
    {
        // Converter as strings de data/hora para objetos DateTime
        $inicio1 = new \DateTime($dadosX['DATAINICIAL']);
        $fim1 = new \DateTime($dadosX['DATAFINAL']);
        $inicio2 = new \DateTime($dadosY['DATAINICIAL']);
        $fim2 = new \DateTime($dadosY['DATAFINAL']);

        // Verificar se há conflito de datas
        if (($inicio1 >= $inicio2 && $inicio1 <= $fim2) ||
            ($fim1 >= $inicio2 && $fim1 <= $fim2) ||
            ($inicio1 <= $inicio2 && $fim1 >= $fim2)
        ) {
            return true;
        }
        return false;
    }

    public function montarDadosTarefa($parametros)
    {
        $intervaloDias = $this->geraItervalo($parametros['DATERANGE'][0], (isset($parametros['DATERANGE'][1]) ? $parametros['DATERANGE'][1] : $parametros['DATERANGE'][0]));

        $parametros['DADOSTAREFAS'] = array();
        foreach ($parametros['EXECUTOR'] as $key => $dados) {
            foreach ($intervaloDias as $dia) {
                $datas = $this->calcularDuracao($dia, $dados['HORAINICIO'], $dados['DURACAO']);
                $dados['DATAINICIAL'] = $datas[0];
                $dados['DATAFINAL'] = $datas[1];
                unset($parametros['EXECUTOR'][$key]);
                array_push($parametros['DADOSTAREFAS'], $dados);
            }
        }
        return $parametros;
    }

    public function montarDadosPosConflito($parametros)
    {
        $parametros['DADOSTAREFAS'] = array();
        foreach ($parametros['EXECUTOR'] as $key => $dados) {
            if ($dados['IGNORARTAREFA'] !== 'S') {
                $datas = $this->calcularDuracao($dados['DATAINICIAL'], $dados['HORAINICIO'], $dados['DURACAO']);
                $dados['DATAINICIAL'] = $datas[0];
                $dados['DATAFINAL'] = $datas[1];
                unset($parametros['EXECUTOR'][$key]);
                array_push($parametros['DADOSTAREFAS'], $dados);
            } else {
                unset($parametros['EXECUTOR'][$key]);
            }
        }
        return $parametros;
    }

    public function calcularDuracao($dataInicial, $horaInicial, $duracao)
    {
        // converter a duração vinda do front para o formato esperado pelo método DateInterval
        $hora = intval(substr($duracao, 0, 2));
        $minuto = intval(substr($duracao, 2, 4));
        $totalMinutos = ($hora * 60) + $minuto;

        $horarioInicial = $dataInicial . " " . $horaInicial;

        $horarioInicial = new \DateTime($horarioInicial);
        $horarioFinal = clone ($horarioInicial);

        $horarioFinal->add(new \DateInterval('PT' . $totalMinutos . 'M'));

        return [$horarioInicial->format('m/d/Y H:i'), $horarioFinal->format('m/d/Y H:i')];
    }
}
