<?php

namespace FM\Service;

use FM\Factory\Fm00040000ModeloAuditoriaFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use FM\Service\FindService;
use FM\Service\FileServerService;
use FM\Service\FuncionariosHierarqService;

class Fm00040000ModeloAuditoriaService
{

    protected $ModeloAuditoriaFactory;
    protected $Environment;
    protected $newCode;
    protected $connection;
    protected $FindService;
    protected $FileServerService;
    protected $fileServerUrl;
    protected $FuncionariosHierarqService;

    public function __construct(Fm00040000ModeloAuditoriaFactory $ModeloAuditoriaFactory, Environment $Environment, NewCode $newCode, EntityManager $entityManager, FindService $FindService, FileServerService $FileServerService, $fileServerUrl, FuncionariosHierarqService $FuncionariosHierarqService)
    {
        $this->connection                   = $entityManager->getConnection();
        $this->ModeloAuditoriaFactory       = $ModeloAuditoriaFactory;
        $this->Environment                  = $Environment;
        $this->newCode                      = $newCode;
        $this->FindService                  = $FindService;
        $this->FileServerService            = $FileServerService;
        $this->fileServerUrl                = $fileServerUrl;
        $this->FuncionariosHierarqService   = $FuncionariosHierarqService;
    }

    public function getTarefasAuditoria()
    {
        $cdoperador = $this->Environment->getCdOperador();
        $listaOperadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($cdoperador);

        $params = array(
            "CDOPERADOR" => $cdoperador
        );

        return $this->ModeloAuditoriaFactory->getTarefasAuditoria($listaOperadores, $params);
    }

    public function inserirTarefaAuditoria($parametros)
    {

        $dados =  $this->montarDadosTarefa($parametros);
        $parametros = $this->validaDiasUteis($dados);
    
        if ($parametros['RECORRENCIA'] !== 'SR') {
            $this->insertTarefa($parametros);
            do {
                if ($parametros['TERMINARECO'] == 'N') {
                    $parametros = $this->calcularRecorrenciaPeriodo($parametros);
                        $this->insertTarefa($parametros);
                }
            } while ($parametros['TERMINARECO'] == 'N');
        } else {
            $this->insertTarefa($parametros);
        }
    }

    public function excluirAuditoria($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $auditorias = $this->FindService->formatarArray($parametros['AUDITORIA']);

            $getResultados = $this->ModeloAuditoriaFactory->getNrResultados($auditorias);
            $arrayResultados = array();

            foreach ($getResultados as $result) {
                array_push($arrayResultados, $result['NRRESULTADOAUDIT']);
            }
            $this->ModeloAuditoriaFactory->deleteCriteriosExecucao($this->FindService->formatarArray($arrayResultados));
            $this->ModeloAuditoriaFactory->deleteTopicosExecucao($auditorias);
            $this->ModeloAuditoriaFactory->deleteAuditoria($auditorias);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function insertTarefa($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $NRORG = $this->Environment->getNrOrg();
            $cdOperador = $this->Environment->getCdOperador();

            foreach ($parametros["EXECUTOR"] as $dados) {
                if ($dados["GERATAREFA"] == 'S') {
                    $dataInicial = new \DateTime($dados['DATAAUDIT']);
                    $nrAuditoria =  $this->newCode->getCodeByTableName("FCM_AUDITORIA", $NRORG, 1, 10, true);

                    $this->ModeloAuditoriaFactory->insertTarefaAuditoria(array(
                        "NRAUDITORIA"           => $nrAuditoria,
                        "NMAUDITORIA"           => $parametros['NMAUDITORIA'],
                        "NRLOCALCLIENTE"        => $dados['NRLOCALCLIENTE'],
                        "NRMODELOAUDIT"         => $parametros['NRMODELOAUDIT'],
                        "NRSEQCONTRATO"         => isset($dados['NRSEQCONTRATO']) ? $dados['NRSEQCONTRATO'] : null,
                        "CDOPERADOR"            => $dados['CDOPERADOR'],
                        "DTHORAAUDIT"           => $dataInicial->format('d/m/Y'),
                        "NRORG"                 => $NRORG,
                        "CDOPERINCLUSAO"        => $cdOperador,
                        "NRORGINCLUSAO"         => $NRORG,
                        "IDOBRIGAOBSERVACAO"    => $parametros['OBSERVACOES'],
                        "IDTIPORECORRENCIA"     => $parametros['RECORRENCIA'],
                        "IDTIPOMARCACAO"        => $dados['IDTIPOMARCACAO'],
                    ));

                    $topicos = $this->ModeloAuditoriaFactory->getTopicosModelo(array(
                        "NRMODELOAUDIT"         => $parametros['NRMODELOAUDIT'],
                    ));

                    foreach ($topicos as $topico) {
                        $nrResultado = $this->newCode->getCodeByTableName("FCM_RESULTADOAUDIT", $NRORG, 1, 10, true);
                        $this->ModeloAuditoriaFactory->inserirTopicosExecucao(array(
                            "NRRESULTADOAUDIT"      => $nrResultado,
                            "NRAUDITORIA"           => $nrAuditoria,
                            "NRMODELOAUDIT"         => $parametros['NRMODELOAUDIT'],
                            "NRTOPICOAUDIT"         => $topico['NRTOPICOAUDIT'],
                            "NRCATEGORIAAUDIT"      => $topico['NRCATEGORIAAUDIT'],
                            "NRORG"                 => $NRORG,
                            "CDOPERINCLUSAO"        => $cdOperador,
                            "NRORGINCLUSAO"         => $NRORG,
                        ));

                        $criterios = $this->ModeloAuditoriaFactory->getCriteriosModelo(array(
                            "NRCATEGORIAAUDIT"         => $topico['NRCATEGORIAAUDIT'],
                        ));

                        foreach ($criterios as $criterio) {
                            $this->ModeloAuditoriaFactory->inserirCriteriosExecucao(array(
                                "NRRELACRESULTCRIT"     => $this->newCode->getCodeByTableName("FCM_RELACRESULTCRIT", $NRORG, 1, 10, true),
                                "NRRESULTADOAUDIT"      => $nrResultado,
                                "NRCRITERIOAUDIT"       => $criterio['NRCRITERIOAUDIT'],
                                "NRTOPICOAUDIT"         => $topico['NRTOPICOAUDIT'],
                                "NRORG"                 => $NRORG,
                                "CDOPERINCLUSAO"        => $cdOperador,
                                "NRORGINCLUSAO"         => $NRORG,
                            ));
                        }
                    }
                }
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    // funções da tab de Modelo

    public function excluirModelo($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $modelo = array(
                "NRMODELOAUDIT" => $parametros['MODELO'],
            );

            $valida = $this->ModeloAuditoriaFactory->validaModeloAuditoria($modelo);
    
            if (intval($valida['CONT']) === 0) {
                $categorias = $this->ModeloAuditoriaFactory->getCategoriaPorModelo($modelo);
                
                foreach ($categorias as $key => $cat) {
                    unset($categorias[$key]);
                    array_push($categorias, intval($cat['NRCATEGORIAAUDIT']));
                }
                
                if (count($categorias) > 0) {
                    $this->ModeloAuditoriaFactory->deleteRelacoesModelo($this->FindService->formatarArray($categorias), $parametros['MODELO']);
                }
                
                $this->ModeloAuditoriaFactory->deleteModelo($modelo);
            }
            $this->connection->commit();
            return $valida['CONT'];
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function excluirDados($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $arrayTopicos = array();
            $arrayCategorias = array();

            $valida = $this->ModeloAuditoriaFactory->validaModeloAuditoria(array(
                "NRMODELOAUDIT" => $parametros['MODELO'],
            ));

            if (intval($valida['CONT']) === 0) {
                foreach ($parametros['DADOS'] as $codigo) {
                    if (substr($codigo, 0, 1) === 'C') {
                        array_push($arrayCategorias, substr($codigo, 1));
                    } else {
                        array_push($arrayTopicos, substr($codigo, 1));
                    }
                }
    
                if (count($arrayTopicos) > 0) {
                    $this->ModeloAuditoriaFactory->deleteRelacaoCatTopicos($this->FindService->formatarArray($arrayTopicos), $parametros['MODELO']);
                }
                
                if (count($arrayCategorias) > 0) {
                    $this->ModeloAuditoriaFactory->deleteRelacaoCatCriterios($this->FindService->formatarArray($arrayCategorias), $parametros['MODELO']);
                    $this->ModeloAuditoriaFactory->deleteRelacaoCatModelo($this->FindService->formatarArray($arrayCategorias), $parametros['MODELO']);
                }
            }
            $this->connection->commit();
            return $valida['CONT'];
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function montarDadosTarefa($parametros)
    {
        $intervaloDias = $this->geraItervalo($parametros['DATERANGE'][0], (isset($parametros['DATERANGE'][1]) ? $parametros['DATERANGE'][1] : $parametros['DATERANGE'][0]));

        

        foreach ($parametros['EXECUTOR'] as $key => $dados) {
            foreach ($intervaloDias as $dia) {
                $dados['DATAAUDIT'] = $dia;
                unset($parametros['EXECUTOR'][$key]);
                array_push($parametros['EXECUTOR'], $dados);
            }
        }
        return $parametros;
    }

    public function geraItervalo($dataInicio, $dataFim)
    {
        // Definir um intervalo de 1 dia
        $fim = new \DateTime($dataFim);
        $intervalo = new \DateInterval('P1D');
        $periodo = new \DatePeriod(new \DateTime($dataInicio), $intervalo, $fim->modify('+1 day'));

        $datas = array();
        foreach ($periodo as $data) {
            array_push($datas, $data->format('m/d/Y'));
        }
        return $datas;
    }

    public function validaDiasUteis($parametros)
    {
        foreach ($parametros['EXECUTOR'] as $key => $exec) {
            $dataInicio = new \DateTime($exec['DATAAUDIT']);
            $exec['DATAINICIALBKP'] = $dataInicio->format('m/d/Y H:i');

            $diaDaSemana = $dataInicio->format('N');
            $dt = $dataInicio->format('m/d/Y');

            // Verificar se a data é um dia útil
            if ($parametros['DIASEMANA'] == '2') {
                if ($diaDaSemana == 6) { // sábado
                    $dataInicio->add(new \DateInterval('P2D')); // soma dois dias para retornar à sexta-feira
                } elseif ($diaDaSemana == 7) { // domingo
                    $dataInicio->add(new \DateInterval('P1D')); // soma um dias para retornar ao último dia útil anterior, que é sexta-feira
                }
                $exec['GERATAREFA'] =  'S';
            } elseif ($parametros['DIASEMANA'] == '3') {
                $exec['GERATAREFA'] = ($diaDaSemana == 6 || $diaDaSemana == 7) ? 'N' : 'S';
                $parametros['EXECUTOR'][$key] = $exec;
            } elseif ($parametros['DIASEMANA'] == '1') {
                $exec['GERATAREFA'] = 'S';
                $parametros['EXECUTOR'][$key] = $exec;
            }

            $dataTermino = (isset($parametros['DTTERMINO'])) ? $parametros['DTTERMINO'] : ((isset($parametros['DATERANGE'][1])) ? $parametros['DATERANGE'][1] : $parametros['DATERANGE'][0]);
            if (new \DateTime($dt) <= new \DateTime($dataTermino)) {
                $exec['DATAAUDIT'] = $dataInicio->format('m/d/Y');
                $parametros['EXECUTOR'][$key] = $exec;
                $parametros['TERMINARECO'] =  'N';
            } else {
                $parametros['TERMINARECO'] =  'S';
                unset($parametros['EXECUTOR'][$key]);
            }
        }

        return $parametros;
    }

    public function calcularRecorrenciaPeriodo($parametros)
    {
        // Definir a frequência da tarefa
        switch ($parametros['RECORRENCIA']) {
            case 'Q':
                $frequencia = new \DateInterval('P14D');
                break;
            case 'M':
                $frequencia = new \DateInterval('P1M');
                break;
            case 'S':
                $frequencia = new \DateInterval('P179D');
                break;
            case 'QP':
                $frequencia = new \DateInterval('P14D');
                break;
            case 'MP':
                $frequencia = new \DateInterval('P1M');
                break;
            case 'SP':
                $frequencia = new \DateInterval('P179D');
                break;
            case 'P':
                $frequencia = new \DateInterval('P'.$parametros['QTDDIAS'].'D');
                break;
        }
        
        // Loop para adicionar as datas das próximas tarefas
        foreach ($parametros['EXECUTOR'] as $key => $executor) {
            $dataInicio = new \DateTime($executor['DATAINICIALBKP']);

            $dataInicio = $dataInicio->add($frequencia);
            $executor['DATAAUDIT'] = $dataInicio->format('m/d/Y');
            $parametros['EXECUTOR'][$key] = $executor;
        }

        return $this->validaDiasUteis($parametros);
    }

    // Funções Tab Modelo
    public function getEstruturaModelo($parametros)
    {
        return $this->ModeloAuditoriaFactory->getEstruturaModelo(array(
            "NRMODELOAUDIT"  => $parametros['NRMODELOAUDIT']
        ));
    }

    public function salvarModelo($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $NRORG = $this->Environment->getNrOrg();
            $cdOperador = $this->Environment->getCdOperador();

            // Valida se a categoria já foi inserida na tabela de relacionamento, ou se é uma nova inserção
            $validar = $this->ModeloAuditoriaFactory->validarCategoria(array(
                "NRCATEGORIAAUDIT"  => $parametros['NRCATEGORIAAUDIT'],
                "NRMODELOAUDIT"         => $parametros['NRMODELOAUDIT'],
            ));

            if (intval($validar['CONT']) === 0) {
                $this->ModeloAuditoriaFactory->insertRelacaoModCategoria(array(
                    "NRRELACMODCATEG"       => $this->newCode->getCodeByTableName("FCM_RELACMODCATEG", $NRORG, 1, 10, true),
                    "NRMODELOAUDIT"         => $parametros['NRMODELOAUDIT'],
                    "NRCATEGORIAAUDIT"      => $parametros['NRCATEGORIAAUDIT'],
                    "NRORG"                 => $NRORG,
                    "CDOPERINCLUSAO"        => $cdOperador,
                    "NRORGINCLUSAO"         => $NRORG
                ));

                
                foreach ($parametros['NRCRITERIOAUDIT'] as $criterios) {
                    $this->ModeloAuditoriaFactory->insertRelacaoCategoriaCriterio(array(
                        "NRRELACCATEGCRIT"     => $this->newCode->getCodeByTableName("FCM_RELACCATEGCRIT", $NRORG, 1, 10, true),
                        "NRCATEGORIAAUDIT"     => $parametros['NRCATEGORIAAUDIT'],
                        "NRMODELOAUDIT"        => $parametros['NRMODELOAUDIT'],
                        "NRCRITERIOAUDIT"      => $criterios,
                        "NRORG"                => $NRORG,
                        "CDOPERINCLUSAO"       => $cdOperador,
                        "NRORGINCLUSAO"        => $NRORG
                    ));
                }
            }

            foreach ($parametros['NRTOPICOAUDIT'] as $topicos) {
                $validaTopico = $this->ModeloAuditoriaFactory->validarTopicos(array(
                    "NRCATEGORIAAUDIT"  => $parametros['NRCATEGORIAAUDIT'],
                    "NRTOPICOAUDIT"  => $topicos,
                    "NRMODELOAUDIT"         => $parametros['NRMODELOAUDIT'],
                ));
        
                if (intval($validaTopico['CONT']) === 0) {
                    $this->ModeloAuditoriaFactory->insertRelacaoCategoriaTopico(array(
                        "NRRELACCATTOPIC"      => $this->newCode->getCodeByTableName("FCM_RELACCATTOPIC", $NRORG, 1, 10, true),
                        "NRCATEGORIAAUDIT"     => $parametros['NRCATEGORIAAUDIT'],
                        "NRMODELOAUDIT"        => $parametros['NRMODELOAUDIT'],
                        "NRTOPICOAUDIT"        => $topicos,
                        "NRORG"                => $NRORG,
                        "CDOPERINCLUSAO"       => $cdOperador,
                        "NRORGINCLUSAO"        => $NRORG
                    ));
                }
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function salvarModeloAuditoria($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $NRORG = $this->Environment->getNrOrg();
            $cdOperador = $this->Environment->getCdOperador();

            $this->ModeloAuditoriaFactory->insertModeloAuditoria(array(
                "NRMODELOAUDIT"        => $this->newCode->getCodeByTableName("FCM_MODELOAUDIT", $NRORG, 1, 10, true),
                "NMMODELOAUDIT"        => $parametros['NMMODELOAUDIT'],
                "NRORG"                => $NRORG,
                "CDOPERINCLUSAO"       => $cdOperador,
                "NRORGINCLUSAO"        => $NRORG
            ));
    
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getModeloAuditoria()
    {
        return $this->ModeloAuditoriaFactory->getModeloAuditoria();
    }
    
    // Funções da tab de Categoria
    public function salvarCategoriaAuditoria($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $NRORG = $this->Environment->getNrOrg();
            $cdOperador = $this->Environment->getCdOperador();

            $this->ModeloAuditoriaFactory->insertCategoriaAuditoria(array(
                "NRCATEGORIAAUDIT"     => $this->newCode->getCodeByTableName("FCM_CATEGORIAAUDIT", $NRORG, 1, 10, true),
                "NMCATEGORIAAUDIT"     => $parametros['NMCATEGORIAAUDIT'],
                "NRORG"                => $NRORG,
                "CDOPERINCLUSAO"       => $cdOperador,
                "NRORGINCLUSAO"        => $NRORG
            ));
    
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getCategoriaAuditoria()
    {
        return $this->ModeloAuditoriaFactory->getCategoriaAuditoria();
    }

    public function excluirCategoria($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $categoria = $this->ModeloAuditoriaFactory->getRelacaoCategorias($this->FindService->formatarArray($parametros['CATEGORIAS']));
            $categoriaExistentes = array();

            foreach ($categoria as $categ) {
                array_push($categoriaExistentes, intval($categ['NRCATEGORIAAUDIT']));
            }

            $categInserir = array_diff($parametros['CATEGORIAS'], $categoriaExistentes);

            if (count($categInserir) > 0) {
                $this->ModeloAuditoriaFactory->deleteCategoria($this->FindService->formatarArray($categInserir));
            }

            $this->connection->commit();
            return count($categoriaExistentes);
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    // Funções Tab Tópicos
    public function salvarTopicosAuditoria($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $NRORG = $this->Environment->getNrOrg();
            $cdOperador = $this->Environment->getCdOperador();

            $this->ModeloAuditoriaFactory->insertTopicosAuditoria(array(
                "NRTOPICOAUDIT"     => $this->newCode->getCodeByTableName("FCM_TOPICOAUDIT", $NRORG, 1, 10, true),
                "NMTOPICOAUDIT"     => $parametros['NMTOPICOAUDIT'],
                "NRORG"             => $NRORG,
                "CDOPERINCLUSAO"    => $cdOperador,
                "NRORGINCLUSAO"     => $NRORG
            ));
    
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getTopicosAuditoria()
    {
        return $this->ModeloAuditoriaFactory->getTopicosAuditoria();
    }

    public function excluirTopicos($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $topicos = $this->ModeloAuditoriaFactory->getRelacaoTopicos($this->FindService->formatarArray($parametros['TOPICOS']));
            $topicosExistentes = array();

            foreach ($topicos as $topic) {
                array_push($topicosExistentes, intval($topic['NRTOPICOAUDIT']));
            }

            $topicInserir = array_diff($parametros['TOPICOS'], $topicosExistentes);

            if (count($topicInserir) > 0) {
                $this->ModeloAuditoriaFactory->deleteTopicos($this->FindService->formatarArray($topicInserir));
            }

            $this->connection->commit();
            return count($topicosExistentes);
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    // Funções da TAB de Critérios
    public function salvarCriteriosAvaliacao($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $NRORG = $this->Environment->getNrOrg();
            $cdOperador = $this->Environment->getCdOperador();

            $this->ModeloAuditoriaFactory->insertCriteriosAvaliacao(array(
                "NRCRITERIOAUDIT"   => $this->newCode->getCodeByTableName("FCM_CRITERIOAUDIT", $NRORG, 1, 10, true),
                "NMCRITERIOAUDIT"   => $parametros[0]['NMCRITERIOAUDIT'],
                "NRORG"             => $NRORG,
                "CDOPERINCLUSAO"    => $cdOperador,
                "NRORGINCLUSAO"     => $NRORG
            ));
    
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getCriteriosAvaliacao()
    {
        return $this->ModeloAuditoriaFactory->getCriteriosAvaliacao();
    }

    public function excluirCriterios($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $criterios = $this->ModeloAuditoriaFactory->getRelacaoCriterios($this->FindService->formatarArray($parametros['CRITERIOS']));
            $criteriosExistentes = array();

            foreach ($criterios as $crit) {
                array_push($criteriosExistentes, intval($crit['NRCRITERIOAUDIT']));
            }

            $critInserir = array_diff($parametros['CRITERIOS'], $criteriosExistentes);

            if (count($critInserir) > 0) {
                $this->ModeloAuditoriaFactory->deleteCriterios($this->FindService->formatarArray($critInserir));
            }
            $this->connection->commit();

            return count($criteriosExistentes);
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    // Funções gerais
    public function excluirRegistros($parametros)
    {
        if (isset($parametros['CRITERIOS'])) {
            return $this->excluirCriterios($parametros);
        } elseif (isset($parametros['TOPICOS'])) {
            return $this->excluirTopicos($parametros);
        } elseif (isset($parametros['CATEGORIAS'])) {
            return $this->excluirCategoria($parametros);
        } elseif (isset($parametros['AUDITORIA'])) {
            return $this->excluirAuditoria($parametros);
        } elseif (isset($parametros['DADOS'])) {
            return $this->excluirDados($parametros);
        } elseif (isset($parametros['MODELO'])) {
            return $this->excluirModelo($parametros);
        }
    }

    public function findLocalAuditoriaCli()
    {
        return $this->ModeloAuditoriaFactory->findLocalAuditoriaCli();
    }

    /* Funções da tela de execução de uma auditoria */

    public function getCategoriasAuditoria($parametros)
    {
        return $this->ModeloAuditoriaFactory->getCategoriasAuditoria(array(
            "NRAUDITORIA"   => $parametros['NRAUDITORIA']
        ));
    }

    public function getTopicosAuditoriaExec($parametros)
    {
        return $this->ModeloAuditoriaFactory->getTopicosAuditoriaExec(array(
            "NRAUDITORIA"        => $parametros['NRAUDITORIA'],
            "NRCATEGORIAAUDIT"   => $parametros['NRCATEGORIAAUDIT'],
        ));
    }

    public function getCriteriosAuditoriaExec($parametros)
    {
        return $this->ModeloAuditoriaFactory->getCriteriosAuditoriaExec(array(
            "NRAUDITORIA"        => $parametros['NRAUDITORIA'],
            "NRTOPICOAUDIT"        => $parametros['NRTOPICOAUDIT'],
        ));
    }

    public function alterarStatusCriterio($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $cdOperador = $this->Environment->getCdOperador();
            $nrorg = $this->Environment->getNrOrg();

            if (!empty($parametros['criteriosN'])) {
                $this->ModeloAuditoriaFactory->updateCriteriosResultado(array(
                    "CDOPERULTATU"             => $cdOperador,
                    "NRORGULTATU"              => $nrorg,
                    "IDREALIZADO"              => 'N',
                ), $this->FindService->formatarArray($parametros['criteriosN']));
            }
            
            if (!empty($parametros['criteriosS'])) {
                $this->ModeloAuditoriaFactory->updateCriteriosResultado(array(
                    "CDOPERULTATU"             => $cdOperador,
                    "NRORGULTATU"              => $nrorg,
                    "IDREALIZADO"              => 'S',
                ), $this->FindService->formatarArray($parametros['criteriosS']));
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function iniciarOuFinalizarAuditoria($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $cdOperador = $this->Environment->getCdOperador();
            $nrorg = $this->Environment->getNrOrg();

            $this->ModeloAuditoriaFactory->updateAuditoria(array(
                "NRAUDITORIA"        => $parametros['NRAUDITORIA'],
                "IDSITUACAOAUDIT"    => $parametros['IDSITUACAOAUDIT'],
                "CDOPERULTATU"       => $cdOperador,
                "NRORGULTATU"        => $nrorg,
            ));

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function salvarDadosObservacao($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $NRORG = $this->Environment->getNrOrg();
            $cdOperador = $this->Environment->getCdOperador();

            foreach ($parametros as $param) {
                // verificar existência de observação para a respectiva categoria
                $existeObservacao = $this->ModeloAuditoriaFactory->existenciaObservacao(array(
                    "NRAUDITORIA"       => $param["NRAUDITORIA"],
                    "NRRELACCATTOPIC"   => $param['NRRELACCATTOPIC'],
                ));

                //Se não existir uma observação registrada na categoria selecionada, será armazenado um novo registro de observação.
                if (count($existeObservacao) > 0) {
                    // Realiza edição do campo Observação para a respectiva categoria
                    $this->ModeloAuditoriaFactory->updateObservacaoAuditoria(array(
                        "NROBSRESULTAUDIT"  => $existeObservacao[0]['NROBSRESULTAUDIT'],
                        "NRAUDITORIA"       => $param["NRAUDITORIA"],
                        "NRRELACCATTOPIC"   => $param['NRRELACCATTOPIC'],
                        "DSOBSERVACAO"      => $param['DSOBSERVACAO'],
                        "NRORG"             => $NRORG,
                        "CDOPERULTATU"      => $cdOperador,
                        "NRORGULTATU"       => $NRORG
                    ));
                } else {
                    $this->ModeloAuditoriaFactory->insertObservacaoAuditoria(array(
                        "NROBSRESULTAUDIT"  => $this->newCode->getCodeByTableName("FCM_OBSRESULTAUDIT", $NRORG, 1, 10, true),
                        "NRAUDITORIA"       => $param["NRAUDITORIA"],
                        "NRRELACCATTOPIC"   => $param['NRRELACCATTOPIC'],
                        "DSOBSERVACAO"      => $param['DSOBSERVACAO'],
                        "NRORG"             => $NRORG,
                        "CDOPERINCLUSAO"    => $cdOperador,
                        "NRORGINCLUSAO"     => $NRORG
                    ));        
                }
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function anexarImagemAuditoria($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $NRORG = $this->Environment->getNrOrg();
            $cdOperador = $this->Environment->getCdOperador();

            $imagens = $this->FileServerService->uploadFiles($parametros['IMAGENS'], 'anexosAuditoria');

            foreach ($imagens as $img) {
                $this->ModeloAuditoriaFactory->anexarImagemAuditoria(array(
                    "NRFOTORESULTAUDIT" => $this->newCode->getCodeByTableName("FCM_FOTORESULTAUDIT", $NRORG, 1, 10, true),
                    "NRAUDITORIA"       => $parametros['NRAUDITORIA'],
                    "NRCATEGORIAAUDIT"  => $parametros['NRCATEGORIAAUDIT'],
                    "NRTOPICOAUDIT"     => $parametros['NRTOPICOAUDIT'],
                    "DSFOTOAUDITORIA"   => $this->fileServerUrl . '/files/' . $img['path'],
                    "NRORG"             => $NRORG,
                    "CDOPERINCLUSAO"    => $cdOperador,
                    "NRORGINCLUSAO"     => $NRORG,
                    "DSTYPE"            => $img['type'],
                    "NRSIZE"            => $img['size']
                ));
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getImagensTopicosAuditoria($parametros)
    {
        return $this->ModeloAuditoriaFactory->getImagensTopicosAuditoria(array(
            "NRAUDITORIA"        => $parametros['NRAUDITORIA'],
            "NRCATEGORIAAUDIT"  => $parametros['NRCATEGORIAAUDIT'],
            "NRTOPICOAUDIT"     => $parametros['NRTOPICOAUDIT'],
        ));
    }

    public function excluirImagemAuditoria($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $this->ModeloAuditoriaFactory->excluirImagemAuditoria(array(
                "NRFOTORESULTAUDIT" => $parametros['NRFOTORESULTAUDIT'],
            ));

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function verificarAlteracoes($parametros)
    {
        try {
            $alteracoesDiferentes = [];

            foreach ($parametros['ALTERACOES'] as $value) {
                 // Verifica se existe no backend
                $haAlteracao = $this->ModeloAuditoriaFactory->verificarAlteracoes([
                    "NRAUDITORIA"     => $value["NRAUDITORIA"],
                    "NRRELACCATTOPIC" => $value['NRRELACCATTOPIC'],
                    "DSOBSERVACAO"    => $value['DSOBSERVACAO'],
                ]);

                // var_dump($haAlteracao);
                // Se a contagem for zero, a observação não existe no backend
                if ($haAlteracao[0]['CONT'] == 0) {
                    return true; // Não há nada no backend
                }
            }

            // Se passou por todas as observações e não encontrou nenhuma alteração
            return false;

        } catch (\Exception $e) {
            throw $e;
        }
    }
}
