<?php

namespace FM\Service;

use FM\Factory\FmModeloPopFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use FM\Service\FindService;
use FM\Service\FileServerService;
use Exception;

class FmModeloPopService
{
    protected $FmModeloPopFactory;
    protected $Environment;
    protected $newCode;
    protected $connection;
    protected $FindService;
    protected $FileServerService;
    protected $fileServerUrl;

    public function __construct(
        FmModeloPopFactory $FmModeloPopFactory,
        Environment $Environment,
        NewCode $newCode,
        EntityManager $entityManager,
        FindService $FindService,
        FileServerService $FileServerService,
        $fileServerUrl
    ) {
        $this->FmModeloPopFactory = $FmModeloPopFactory;
        $this->Environment = $Environment;
        $this->newCode = $newCode;
        $this->connection = $entityManager->getConnection();
        $this->FindService = $FindService;
        $this->FileServerService = $FileServerService;
        $this->fileServerUrl = $fileServerUrl;
    }

    public function getModeloPop()
    {
        return $this->FmModeloPopFactory->getModeloPop();
    }
    public function salvarModeloPop($params)
{
    $this->connection->beginTransaction();
    try {
        $nrorg = $this->Environment->getNrOrg();
        $operador = $this->Environment->getCdOperador();

        if (!isset($params["NRMODELOPOP"]) || empty($params["NRMODELOPOP"])) {
            $NRMODELOPOP = $this->newCode->getCodeByTableName("FCM_MODELOPOP", $nrorg, 1, 10, true);
            $paramModeloPop = array(
                "NRMODELOPOP" => $NRMODELOPOP,
                "NMMODELOPOP" => $params["NMMODELOPOP"],
                "LINKMODELOPOP" => $params["LINKMODELOPOP"],
                "DESCMODELOPOP" => $params["DESCMODELOPOP"],
                "IDATIVO" => 'S',
                "NRORG" => $nrorg,
                "NRORGINCLUSAO" => $nrorg,
                "CDOPERINCLUSAO" => $operador
            );
            $this->FmModeloPopFactory->inserirModeloPop($paramModeloPop);
        } else {
            $paramModeloPop = array(
                "NRMODELOPOP" => $params["NRMODELOPOP"],
                "NMMODELOPOP" => $params["NMMODELOPOP"],
                "LINKMODELOPOP" => $params["LINKMODELOPOP"],
                "DESCMODELOPOP" => $params["DESCMODELOPOP"],
                "CDOPERULTATU" => $operador,
                "NRORGULTATU" => $nrorg
            );
            $this->FmModeloPopFactory->atualizarModeloPop($paramModeloPop);
        }
        if (isset($params['ARQUIVO']) && is_array($params['ARQUIVO']) && count($params['ARQUIVO']) > 0) {
            $file = $this->FileServerService->uploadFiles($params["ARQUIVO"], 'anexo_modelo_documento_cda');
            $file[0]["size"] = $file[0]["size"] / 1024;
        } else {
            $file = [[
                "path" => isset($params["DSCAMINHOARQ"]) ? $params["DSCAMINHOARQ"] : '',
                "size" => isset($params["VRTAMANHO"]) ? $params["VRTAMANHO"] : 0,
                "name" => isset($params["NMARQUIVO"]) ? $params["NMARQUIVO"] : '',
                "type" => isset($params["DSTIPO"]) ? $params["DSTIPO"] : '',
            ]];
        }

        if (!isset($params["NRANEXOMODELOPOP"])) {
            $NRANEXOMODELOPOP = $this->newCode->getCodeByTableName('FCM_ANEXOMODELOPOP', $nrorg, 1, 10, true);
            $paramsAnexo = array(
                "NRANEXOMODELOPOP" => $NRANEXOMODELOPOP,
                "NRMODELOPOP" => $paramModeloPop["NRMODELOPOP"], 
                "NMARQUIVO" => $file[0]["name"],
                "DSCAMINHOARQ" => $this->fileServerUrl . "/files/" . $file[0]["path"],
                "NRORG" => $nrorg,
                "NRORGINCLUSAO" => $nrorg,
                "IDATIVO" => 'S',
                "CDOPERINCLUSAO" => $operador,
                "DSTIPO" => $file[0]["type"],
                "VRTAMANHO" => $file[0]["size"],
            );
            $this->FmModeloPopFactory->inserirAnexoModeloPop($paramsAnexo);
        } else {
            $paramsAnexo = array(
                "NRANEXOMODELOPOP" => $params["NRANEXOMODELOPOP"],
                "NRMODELOPOP" => $paramModeloPop["NRMODELOPOP"],
                "NMARQUIVO" => $file[0]["name"],
                "DSCAMINHOARQ" => $this->fileServerUrl . "/files/" . $file[0]["path"],
                "NRORG" => $nrorg,
                "DSTIPO" => $file[0]["type"],
                "VRTAMANHO" => $file[0]["size"],
                "IDATIVO" => 'S',
                "NRORGULTATU" => $nrorg,
                "CDOPERULTATU" => $operador
            );
            $this->FmModeloPopFactory->atualizarAnexoModeloPop($paramsAnexo);
        }

        $this->connection->commit();
    } catch (\Exception $e) {
        $this->connection->rollBack();
        throw $e;
    }
}

    public function deletarAnexoPop($params)
{
    $this->connection->beginTransaction();
    try {
        foreach ($params as $anexo) {
            if (!isset($anexo['NRANEXOMODELOPOP']) || empty($anexo["NRANEXOMODELOPOP"])) {
                continue;
            }

            $dataParam = array(
                "NRANEXOMODELOPOP" => $anexo["NRANEXOMODELOPOP"],
            );

            if (!empty($anexo['DSCAMINHOARQ'])) {
                $this->FileServerService->deleteFile($anexo['DSCAMINHOARQ']);
            }

            if ($dataParam["NRANEXOMODELOPOP"] !== null) {
                $this->FmModeloPopFactory->deletarAnexoPop($dataParam);
            }
        }
        $this->connection->commit();
    } catch (\Exception $e) {
        $this->connection->rollback();
        throw $e;
    }
}
    public function excluirModeloPop($parametros)
    {
        $this->connection->beginTransaction(); 
        try {
            foreach ($parametros['ROW'] as $nr) {
                $param = array(
                    "NRMODELOPOP" => $nr
                );
                $anexos = $this->getModeloPop();
                $this->deletarAnexoPop($anexos["data"]);
                $this->FmModeloPopFactory->excluirModeloPop($param);
            }
        
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    
}
