<?php

namespace FM\Service;

use FM\Factory\FmtAtivosInsumosFactory;
use FM\Service\FileServerService;
use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;

class FmtAtivosInsumosService
{
    protected $AtivosInsumosFactory;
    protected $FileServerService;
    protected $connection;
    protected $Environment;
    protected $newCode;
    protected $fileServerUrl;

    public function __construct(FmtAtivosInsumosFactory $AtivosInsumosFactory, FileServerService $FileServerService, EntityManager $entityManager, Environment $Environment, NewCode $newCode, $fileServerUrl)
    {
        $this->AtivosInsumosFactory = $AtivosInsumosFactory;
        $this->FileServerService = $FileServerService;
        $this->connection = $entityManager->getConnection();
        $this->Environment = $Environment;
        $this->newCode = $newCode;
        $this->fileServerUrl = $fileServerUrl;
    }

    /*
     * TAB ATIVOS
     */

    public function getAtivos()
    {
        return $this->AtivosInsumosFactory->getAtivos();
    }

    public function salvarAtivo($params)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->Environment->getNrorg();
            $operador = $this->Environment->getCdOperador();
            if ($params["NRROW"] == null) {
                $NREQUIPAMENTO = $this->newCode->getCodeByTableName('FCM_EQUIPAMENTO', 20, 1, 10, true);

                $parametroAtivo = array(
                    "NREQUIPAMENTO"     => $NREQUIPAMENTO,
                    "NMEQUIPAMENTO"     => $params["NMEQUIPAMENTO"],
                    "CDEQUIPAMENTO"     => $params["CDEQUIPAMENTO"],
                    "NRCATEGORIAEQUIP"  => $params["NRCATEGORIAEQUIP"],
                    "IDSITUACAO"        => $params["IDSITUACAO"],
                    "NRORG"             => $nrorg,
                    "CDOPERINCLUSAO"    => $operador,
                    "NRORGINCLUSAO"     => $nrorg
                );

                $this->AtivosInsumosFactory->insertAtivo($parametroAtivo);

                foreach ($params["ESCOPOS"] as $escopo) {
                    $RELACESCOPOSEQUIP = $this->newCode->getCodeByTableName('FCM_RELACESCOPOSEQUIP', 20, 1, 10, true);

                    $parametroEscopo = array(
                        "RELACESCOPOSEQUIP" => $RELACESCOPOSEQUIP,
                        "NREQUIPAMENTO"     => $NREQUIPAMENTO,
                        "NRESCOPOMANUT"     => $escopo,
                        "NRORG"             => $nrorg
                    );

                    $this->AtivosInsumosFactory->insertRelacEscopoEquip($parametroEscopo);
                }
            } else {
                $NREQUIPAMENTO = $params["NRROW"];

                $parametroAtivo = array(
                    "NREQUIPAMENTO"     => $NREQUIPAMENTO,
                    "NMEQUIPAMENTO"     => $params["NMEQUIPAMENTO"],
                    "CDEQUIPAMENTO"     => $params["CDEQUIPAMENTO"],
                    "NRCATEGORIAEQUIP"  => $params["NRCATEGORIAEQUIP"],
                    "IDSITUACAO"        => $params["IDSITUACAO"],
                    "CDOPERULTATU"      => $operador,
                    "NRORGULTATU"       => $nrorg
                );

                $this->AtivosInsumosFactory->updateAtivo($parametroAtivo);

                foreach ($params["ESCOPOS"] as $escopo) {
                    $parametroRelacionamento = array(
                        "NREQUIPAMENTO" => $NREQUIPAMENTO,
                        "NRESCOPOMANUT" => $escopo
                    );
                    $numRelacionamentos = $this->AtivosInsumosFactory->getNumRelacEquipEscopo($parametroRelacionamento);

                    if ($numRelacionamentos == 0) {
                        $RELACESCOPOSEQUIP = $this->newCode->getCodeByTableName('FCM_RELACESCOPOSEQUIP', 20, 1, 10, true);

                        $parametroEscopo = array(
                            "RELACESCOPOSEQUIP" => $RELACESCOPOSEQUIP,
                            "NREQUIPAMENTO"     => $NREQUIPAMENTO,
                            "NRESCOPOMANUT"     => $escopo,
                            "NRORG"             => $nrorg
                        );

                        $this->AtivosInsumosFactory->insertRelacEscopoEquip($parametroEscopo);
                    }
                }
            }

            $this->connection->commit();
        } catch (\Exception $ex) {
            $this->connection->rollback();
        }
    }

    public function deleteAtivos($params)
    {
        $parametroAtivo = implode(',', $params["ROWS"]);

        $this->AtivosInsumosFactory->deleteAtivos($parametroAtivo);
    }

    public function insertFotosAtivo($params)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->Environment->getNrorg();

            foreach ($params["FOTOS"] as $foto) {
                $img = $this->FileServerService->uploadFoto([$foto], "foto_manutencao_fmt");

                $NRIMGEQUIPAMENTO = $this->newCode->getCodeByTableName('FCM_IMGEQUIPAMENTO', 20, 1, 10, true);
                $parametroImagem = array(
                    "NRIMGEQUIPAMENTO"  => $NRIMGEQUIPAMENTO,
                    "NREQUIPAMENTO"     => $params["NREQUIPAMENTO"],
                    "DSCAMINHO"         => $this->fileServerUrl . "/files/" . $img["path"],
                    "NRORG"             => $nrorg,
                    "DSTYPE"            => $img["type"],
                    "NRSIZE"            => $img["size"]
                );

                $this->AtivosInsumosFactory->insertImgAtivo($parametroImagem);
            }

            $this->connection->commit();
        } catch (\Exception $ex) {
            $this->connection->rollback();
        }
    }

    public function getFotosAtivo($params)
    {
        $parametroFotos = array(
            "NREQUIPAMENTO" => $params["NREQUIPAMENTO"]
        );

        return $this->AtivosInsumosFactory->getFotosAtivo($parametroFotos);
    }

    public function deleteFotoAtivo($params)
    {
        $parametroFoto = array(
            "NRIMGEQUIPAMENTO" => $params["NRIMGEQUIPAMENTO"]
        );

        return $this->AtivosInsumosFactory->deleteFotoAtivo($parametroFoto);
    }

    public function getCategEquip()
    {
        return $this->AtivosInsumosFactory->getCategEquip();
    }

    public function getEscopos()
    {
        return $this->AtivosInsumosFactory->getEscopos();
    }

    /*
     * TAB CATEGORIAS
     */

    public function getCategorias()
    {
        $data = $this->AtivosInsumosFactory->getCategorias();

        foreach ($data["data"] as &$row) {
            if (isset($row["NRCATEGORIAMANUT"])) {
                foreach ($row["NRCATEGORIAMANUT"] as &$NR) {
                    if($NR == "") {
                        $NR = [];
                    } else {
                        $NR = (int) $NR;
                    }
                }
            }
        }

        return $data;
    }

    public function salvarCategoria($params)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->Environment->getNrorg();
            $operador = $this->Environment->getCdOperador();
            if ($params["NRROW"] == null) {
                $NRCATEGORIAEQUIP = $this->newCode->getCodeByTableName("FCM_CATEGORIAEQUIP", 20, 1, 10, true);

                $parametroCategoria = array(
                    "NRCATEGORIAEQUIP"  => $NRCATEGORIAEQUIP,
                    "NMCATEGORIAEQUIP"  => $params["NMCATEGORIAEQUIP"],
                    "NRORG"             => $nrorg,
                    "CDOPERINCLUSAO"    => $operador,
                    "NRORGINCLUSAO"     => $nrorg
                );

                $this->AtivosInsumosFactory->insertCategoria($parametroCategoria);

                foreach ($params["CATEGORIAS"] as $categoria) {
                    $NRRELCATEGORIAEQUIPMANUT = $this->newCode->getCodeByTableName("FCM_RELCATEGORIAEQUIPMANUT", 20, 1, 10, true);

                    $parametroManutencao = array(
                        "NRRELCATEGORIAEQUIPMANUT"  => $NRRELCATEGORIAEQUIPMANUT,
                        "NRCATEGORIAEQUIP"          => $NRCATEGORIAEQUIP,
                        "NRCATEGORIAMANUT"          => $categoria,
                        "NRORG"                     => $nrorg
                    );

                    $this->AtivosInsumosFactory->insertRelCatManut($parametroManutencao);
                }
            } else {
                $NRCATEGORIAEQUIP = $params["NRROW"];

                $parametroCategoria = array(
                    "NRCATEGORIAEQUIP"  => $NRCATEGORIAEQUIP,
                    "NMCATEGORIAEQUIP"  => $params["NMCATEGORIAEQUIP"],
                    "CDOPERULTATU"      => $operador,
                    "NRORGULTATU"       => $nrorg
                );

                $this->AtivosInsumosFactory->updateCategoria($parametroCategoria);

                $this->AtivosInsumosFactory->deleteRelCatEquipManut($NRCATEGORIAEQUIP);

                foreach ($params["CATEGORIAS"] as $categoria) {
                    $NRRELCATEGORIAEQUIPMANUT = $this->newCode->getCodeByTableName("FCM_RELCATEGORIAEQUIPMANUT", 20, 1, 10, true);

                    $parametroManutencao = array(
                        "NRRELCATEGORIAEQUIPMANUT"  => $NRRELCATEGORIAEQUIPMANUT,
                        "NRCATEGORIAEQUIP"          => $NRCATEGORIAEQUIP,
                        "NRCATEGORIAMANUT"          => $categoria,
                        "NRORG"                     => $nrorg
                    );

                    $this->AtivosInsumosFactory->insertRelCatManut($parametroManutencao);
                }
            }

            $this->connection->commit();
        } catch (\Exception $ex) {
            $this->connection->rollback();
        }
    }

    public function editarAgrupamentos($rows) {
        $this->connection->beginTransaction();
        
        try {
            $nrorg = $this->Environment->getNrorg();
            $operador = $this->Environment->getCdOperador();
        
            foreach ($rows["ROWS"] as $row) {
                $NRCATEGORIAEQUIP = $row["NRCATEGORIAEQUIP"];
                
                $parametroCategoria = array(
                    "NRCATEGORIAEQUIP"  => $NRCATEGORIAEQUIP,
                    "NMCATEGORIAEQUIP"  => $row["NMCATEGORIAEQUIP"],
                    "CDOPERULTATU"      => $operador,
                    "NRORGULTATU"       => $nrorg,
                );
        
                $this->AtivosInsumosFactory->updateCategoria($parametroCategoria);
        
                $this->AtivosInsumosFactory->deleteRelCatEquipManut($NRCATEGORIAEQUIP);
        
                foreach ($row["NRCATEGORIAMANUT"] as $NRCATEGORIAMANUT) {
                    $NRRELCATEGORIAEQUIPMANUT = $this->newCode->getCodeByTableName("FCM_RELCATEGORIAEQUIPMANUT", 20, 1, 10, true);
        
                    $parametroManutencao = array(
                        "NRRELCATEGORIAEQUIPMANUT"  => $NRRELCATEGORIAEQUIPMANUT,
                        "NRCATEGORIAEQUIP"          => $NRCATEGORIAEQUIP,
                        "NRCATEGORIAMANUT"          => $NRCATEGORIAMANUT,
                        "NRORG"                     => $nrorg
                    );
        
                    $this->AtivosInsumosFactory->insertRelCatManut($parametroManutencao);
                }
            }
            $this->connection->commit();
        } catch (\Exception $ex) {
            $this->connection->rollback();
        }

    }

    public function deleteCategorias($params)
    {
        $parametro = implode(',', $params["ROWS"]);
        $this->AtivosInsumosFactory->deleteRelCatEquipManut($parametro);
        $this->AtivosInsumosFactory->deleteCategorias($parametro);
    }

    public function getCategManutencao()
    {
        return $this->AtivosInsumosFactory->getCategManutencao();
    }

    public function getEquip($parametro)
    {
        $params = array(
            "NRCATEGORIAEQUIP" => $parametro["NRCATEGORIAEQUIP"]
        );
        return $this->AtivosInsumosFactory->getEquip($params);
    }

    /*
     * TAB INSUMOS
     */

    public function getInsumos()
    {
        return $this->AtivosInsumosFactory->getInsumos();
    }

    public function salvarInsumo($params)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->Environment->getNrorg();
            $operador = $this->Environment->getCdOperador();
            if ($params["NRROW"] == null) {
                $NRINSUMO = $this->newCode->getCodeByTableName("FCM_INSUMO", 20, 1, 10, true);

                $parametroInsumo = array(
                    "NRINSUMO"          => $NRINSUMO,
                    "NMINSUMO"          => $params["NMINSUMO"],
                    "CDINSUMO"          => $params["CDINSUMO"],
                    "CDPRODUTO"         => $params["CDPRODUTO"],
                    "NRUNIDADEMEDIDA"   => $params["NRUNIDADEMEDIDA"],
                    "NRORG"             => $nrorg,
                    "CDOPERINCLUSAO"    => $operador,
                    "NRORGINCLUSAO"     => $nrorg
                );

                $this->AtivosInsumosFactory->insertInsumo($parametroInsumo);
            } else {
                $parametroInsumo = array(
                    "NRINSUMO"          => $params["NRROW"],
                    "NMINSUMO"          => $params["NMINSUMO"],
                    "CDINSUMO"          => $params["CDINSUMO"],
                    "CDPRODUTO"         => $params["CDPRODUTO"],
                    "NRUNIDADEMEDIDA"   => $params["NRUNIDADEMEDIDA"],
                    "CDOPERULTATU"      => $operador,
                    "NRORGULTATU"       => $nrorg
                );

                $this->AtivosInsumosFactory->updateInsumo($parametroInsumo);
            }

            $this->connection->commit();
        } catch (\Exception $ex) {
            $this->connection->rollback();
        }
    }

    public function deleteInsumos($params)
    {
        $parametro = implode(',', $params["ROWS"]);
        $this->AtivosInsumosFactory->deleteInsumos($parametro);
    }

    public function insertFotosInsumo($params)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->Environment->getNrorg();

            foreach ($params["FOTOS"] as $foto) {
                $img = $this->FileServerService->uploadFoto([$foto], "foto_manutencao_fmt");

                $NRIMGINSUMO = $this->newCode->getCodeByTableName('FCM_IMGINSUMO', 20, 1, 10, true);
                $parametroImagem = array(
                    "NRIMGINSUMO"   => $NRIMGINSUMO,
                    "NRINSUMO"      => $params["NRINSUMO"],
                    "DSCAMINHO"     => $this->fileServerUrl . "/files/" . $img["path"],
                    "NRORG"         => $nrorg,
                    "DSTYPE"        => $img["type"],
                    "NRSIZE"        => $img["size"]
                );

                $this->AtivosInsumosFactory->insertImgInsumo($parametroImagem);
            }

            $this->connection->commit();
        } catch (\Exception $ex) {
            $this->connection->rollback();
        }
    }

    public function getFotosInsumo($params)
    {
        $parametroFotos = array(
            "NRINSUMO" => $params["NRINSUMO"]
        );

        return $this->AtivosInsumosFactory->getFotosInsumo($parametroFotos);
    }

    public function deleteFotoInsumo($params)
    {
        $parametroFoto = array(
            "NRIMGINSUMO" => $params["NRIMGINSUMO"]
        );

        return $this->AtivosInsumosFactory->deleteFotoInsumo($parametroFoto);
    }

    public function getUniMedida()
    {
        return $this->AtivosInsumosFactory->getUniMedida();
    }
}
