import { Utilities } from '@/util/Utilities';
import {
    DialogService,
    LoadingService,
    TextInput,
} from '@zeedhi/common';
import { Http, IEventParam, Metadata } from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class EPIsController {
    public addNewRow({ component }: IEventParam<TekGrid>) {
        const newId = 0;
        component.addNewRow({ NREPI: newId, IDATIVO: 'S' }, 'start');
        setTimeout(() => {
            const nameInput = Metadata.getInstance<TextInput>(`NMEPI_editable_${newId}`);
            nameInput.setFocus();
        }, 100);
    }

    public salvarEPI({ component, event }: IEventParam<TekGrid>) {
        event?.preventDefault();
        if (Utilities.validateGrid(component)) {
            LoadingService.show();
            Http.post('/salvarEPI', {
                ROW: component.getEditedRows(),
            })
            .then(() => {
                Utilities.showSuccessMessage();
            })
            .catch((error: any) => {
                Utilities.showErrorMessage(error);
            })
            .finally(() => {
                component.cancelEditedRows();
                LoadingService.hide();
            });
        }
    }

    public messageConfirmDelete({ event }: any) {
        Utilities.showConfirmDeleteMessage(event, 'EPIsController.excluirEPI', 'Excluir EPI', 'Deseja realmente excluir os registros selecionados?');
    }

    public excluirEPI({ event }: IEventParam<TekGrid>) {
        event?.preventDefault();
        const grid = Metadata.getInstance<TekGrid>('TekGrid_template_gridEpis');
        LoadingService.show();
        Http.post('/excluirEPI', {
            ROW: grid.selectedRows.map((element: any) => element.NREPI),
        }).then(() => {
            grid.datasource.get();
            Utilities.showSuccessMessage();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            grid.selectedRows = [];
            LoadingService.hide();
            DialogService.hide();
        });
    }
}
