import { Utilities } from '@/util/Utilities';
import { DialogService, GridEditable, LoadingService } from '@zeedhi/common';
import { Http, IEventParam, Metadata } from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class Fm0000000EquipesController {
    public visibilidadeFrameOperadores: boolean = false;

    public visibilidadeFrameEquipes: boolean = true;

    public currentRow: any;

    public tituloHeaderFrameOperadoresEquipes: String = '';

    public selectEditavel: boolean = false;

    public rowClick({ row, component }: any) {
        if (component.editing === false) {
            this.currentRow = row;
            this.tituloHeaderFrameOperadoresEquipes = row.NMEQUIPE;
            this.visibilidadeFrameEquipes = false;
            this.visibilidadeFrameOperadores = true;
            this.setFilterOperadorEquipe();
        }
    }

    public voltarGridEquipes() {
        this.visibilidadeFrameOperadores = false;
        this.visibilidadeFrameEquipes = true;
    }

    public addNewRow({ component }: IEventParam<TekGrid>) {
        const newId = 0;
        const newRow = {
            NREQUIPE: newId,
            NMEQUIPE: '',
        };
        component.addNewRow(newRow, 'start');
    }

    public async saveEditingAddNewRow({ component, event }: IEventParam<GridEditable>) {
        event?.preventDefault();
        await component.saveEditedRows();
        component.editingNewRows = false;
    }

    public salvarEquipes({ component, event }: IEventParam<TekGrid>) {
        event?.preventDefault();
        if (Utilities.validateGrid(component)) {
            LoadingService.show();
            Http.post('/salvarEquipes', {
                ROW: component.getEditedRows(),
            }).then(() => {
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                component.cancelEditedRows();
                LoadingService.hide();
            });
        }
    }

    public excluirEquipes() {
        const grid = Metadata.getInstance<TekGrid>('TekGrid_template_grid-equipes');
        DialogService.hide();
        LoadingService.show();
        Http.post('/excluirEquipes', {
            ROW: grid.selectedRows.map((element: any) => element.NREQUIPE),
        }).then(() => {
            grid.datasource.get();
            Utilities.showSuccessMessage();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            grid.selectedRows = [];
            LoadingService.hide();
        });
    }

    public setFilterOperadorEquipe() {
        const gridOperador = Metadata.getInstance<TekGrid>('TekGrid_template_grid-operadores');
        gridOperador.datasource.setFilter({
            EQUIPE: this.currentRow.NREQUIPE,
        });
    }

    public addNewRowOperador({ component }: IEventParam<TekGrid>) {
        this.selectEditavel = true;
        const newId = 0;
        component.addNewRow({ NRRELACEQUIPEOPER: newId }, 'start');
    }

    public salvarOperadorEquipe({ component, event }: IEventParam<TekGrid>) {
        event?.preventDefault();
        if (Utilities.validateGrid(component)) {
            LoadingService.show();
            const newRow = component.getEditedRows();
            Http.post('/relacionarOperadorEquipe', {
                OPERADOR: newRow[0].CDOPERADOR,
                NREQUIPE: this.currentRow.NREQUIPE,
                IDATIVO: this.currentRow.IDATIVO,
            }).then(() => {
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                this.selectEditavel = false;
                component.cancelEditedRows();
                LoadingService.hide();
            });
        }
    }

    public excluirOperadorEquipeDialog() {
        DialogService.show({
            name: 'apagar-operador-dialog',
            title: 'Excluir o operador',
            type: 'warning',
            text: 'Deseja Realmente excluir o operador dessa equipe?',
            buttons: [
                {
                    name: 'no', component: 'ZdButton', label: 'Nao', events: { click: () => DialogService.hide() }, outline: true,
                },
                {
                    name: 'yes', component: 'ZdButton', label: 'Sim', events: { click: () => this.excluirOperadorEquipe() },
                },
            ],
        });
    }

    public excluirEquipeDialog() {
        DialogService.show({
            name: 'apagar-equipe-dialog',
            title: 'Excluir Equipe',
            type: 'warning',
            text: 'Deseja Realmente excluir a Equipe?',
            buttons: [
                {
                    name: 'no', component: 'ZdButton', label: 'Nao', events: { click: () => DialogService.hide() }, outline: true,
                },
                {
                    name: 'yes', component: 'ZdButton', label: 'Sim', events: { click: () => this.excluirEquipes() },
                },
            ],
        });
    }

    public excluirOperadorEquipe() {
        const grid = Metadata.getInstance<TekGrid>('TekGrid_template_grid-operadores');
        LoadingService.show();
        DialogService.hide();
        Http.post('/excluirOperadorEquipe', {
            ROW: grid.selectedRows.map((element: any) => element.NRRELACEQUIPEOPER),
        }).then(() => {
            grid.datasource.get();
            Utilities.showSuccessMessage();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            grid.selectedRows = [];
            LoadingService.hide();
        });
    }
}
