import {
    Modal, ModalService, TextInput, LoadingService, DialogService, Form, IGridEvent,
    FileInput,
} from '@zeedhi/common';
import {
    IEventParam, Metadata, Http,
} from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { Utilities } from '@/util/Utilities';
import { RichText } from '@zeedhi/zd-richtext';

export class FmModeloPopController {
    private exibirInformacoesModeloModal!: Modal;

    public currentRow: any;

    private tituloModal: string = '';

    public resposta: string = '';

    public nrRowAtual: any;

    public rowDeleteAnexo: any = {};

    public habilitarBotao = true;

    public async createModalPop() {
        this.exibirInformacoesModeloModal = await ModalService.createFromJsonPost('', { jsonPath: 'fm_modelopop/fm_modelopop_modal' });
    }

    public async limparCampos() {
        Metadata.getInstance<Form>('formCadastroModelo').clearForm();
        Metadata.getInstance<Form>('formArquivoCadastroModelo').clearForm();
    }

    public async exibirModalNovoCadastro() {
        await this.exibirInformacoesModeloModal.show();
        this.tituloModal = 'Cadastro Modelo';
        this.limparCampos();
    }

    public async exibirModalCadastroEdicao({ row }: IGridEvent) {
        this.tituloModal = 'Editar Modelo';
        await this.exibirInformacoesModeloModal.show();

        Metadata.getInstance<TextInput>('NMMODELOPOP').value = row.NMMODELOPOP;
        Metadata.getInstance<RichText>('DESCMODELOPOP').value = row.DESCMODELOPOP;
        Metadata.getInstance<TextInput>('LINKMODELOPOP').value = row.LINKMODELOPOP;
        Metadata.getInstance<TextInput>('NRMODELOPOP').value = row.NRMODELOPOP;
        Metadata.getInstance<TextInput>('NRANEXOMODELOPOP').value = row.NRANEXOMODELOPOP;
        Metadata.getInstance<TextInput>('NMARQUIVO').value = row.NMARQUIVO;
        Metadata.getInstance<FileInput>('ARQUIVO').isVisible = true;
    }

    public destruirModal() {
        this.exibirInformacoesModeloModal?.destroy();
    }

    public async fecharModalPop() {
        this.exibirInformacoesModeloModal.hide();
    }

    public async salvarModeloPop() {
        const grid = Metadata.getInstance<TekGrid>('TekGrid_template_grid-modelopop');
        const DESCMODELOPOP = Metadata.getInstance<RichText>('DESCMODELOPOP').value;
        const LINKMODELOPOP = Metadata.getInstance<TextInput>('LINKMODELOPOP').value;
        const NMMODELOPOP = Metadata.getInstance<TextInput>('NMMODELOPOP').value;
        const NRMODELOPOP = Metadata.getInstance<TextInput>('NRMODELOPOP').value;
        const NRANEXOMODELOPOP = Metadata.getInstance<FileInput>('NRANEXOMODELOPOP').value;
        const file = (await Utilities.buildArrayToUpload('ARQUIVO'));
        const camposValidos = (DESCMODELOPOP === '<p></p>' && LINKMODELOPOP === null && file.length <= 0);

        if (camposValidos) {
            Utilities.alertMessage('Preencha pelo menos um modelo de procedimento padrão.', 'error');
            return;
        }

        const postData = {
            NMMODELOPOP,
            NRMODELOPOP,
            NRANEXOMODELOPOP,
            DESCMODELOPOP,
            LINKMODELOPOP,
            ARQUIVO: file,
        };

        LoadingService.show();
        try {
            await Http.post('/salvarModeloPop', postData);
            Utilities.showSuccessMessage();
            grid.datasource.get();
            this.fecharModalPop();
        } catch (error) {
            Utilities.showErrorMessage(error);
        } finally {
            LoadingService.hide();
        }
    }

    public messageConfirmDelete({ event }: any) {
        Utilities.showConfirmDeleteMessage(event, 'FmModeloPopController.aposConfirmarDelete', 'Excluir Modelo', 'Deseja realmente excluir os registros selecionados?');
    }

    public async aposConfirmarDelete({ event }: IEventParam<TekGrid>) {
        event?.preventDefault();
        const grid = Metadata.getInstance<TekGrid>('TekGrid_template_grid-modelopop');
        const selectedRows = grid.selectedRows.map((element: any) => element.NRMODELOPOP);
        LoadingService.show();
        try {
            await Http.post('/deletarAnexoPop', {
                row: this.rowDeleteAnexo,
            });
            await Http.post('/excluirModeloPop', { ROW: selectedRows });
            grid.datasource.get();
            Utilities.showSuccessMessage();
        } catch (error) {
            Utilities.showErrorMessage(error);
        } finally {
            grid.selectedRows = [];
            LoadingService.hide();
            DialogService.hide();
        }
    }
}
