import {
    DialogService, Form, LoadingService, Modal, ModalService,
} from '@zeedhi/common';
import { Http, IEventParam, Metadata } from '@zeedhi/core';
import { TekGrid, TekTreeGrid } from '@zeedhi/teknisa-components-common';
import { Utilities } from '@/util/Utilities';

export class FmtAtivosInsumosCategoriasController {
    private modalSalvarCategoria!: Modal;

    public modalVisualizar!: Modal;

    public currentRow: any = {};

    public async abrirModal() {
        this.modalSalvarCategoria = await ModalService.createFromJsonPost('', { jsonPath: '/fmt_ativosInsumos/modal/fmt_ativos_insumos_salvar_categoria' });
        this.modalSalvarCategoria.show();
    }

    public async abrirModalAdicionar() {
        await this.abrirModal();
    }

    public async abrirModalVisualizacao() {
        this.modalVisualizar = await ModalService.createFromJsonPost('', { jsonPath: '/fmt_ativosInsumos/modal/fmt_ativos_insumos_visualizar_agrupamentos' });
        this.modalVisualizar.show();
    }

    public async fecharModalVisualizacao() {
        this.modalVisualizar.hide();
        this.modalVisualizar.destroy();
    }

    public fecharModal() {
        this.modalSalvarCategoria.hide();
        this.modalSalvarCategoria?.destroy();
    }

    public salvarCategoria() {
        const form = Metadata.getInstance<Form>('form-salvar-categoria');
        const grid = Metadata.getInstance<TekGrid>('tek-grid-categorias');

        if (Utilities.validateForm(form)) {
            LoadingService.show();
            Http.post('/salvarCategoria', {
                NRROW: form.value.NRROW,
                NMCATEGORIAEQUIP: form.value.InputNomeCategoria,
                CATEGORIAS: form.value.SelectMultipleCategoriaManutencao,
            }).then(() => {
                Utilities.showSuccessMessage();
                grid.datasource.get();
                this.fecharModal();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public editarAgrupamentos({ component }: IEventParam<TekGrid>) {
        if (Utilities.validateGrid(component)) {
            LoadingService.show();
            Http.post('/editarAgrupamentos', {
                ROWS: component.getEditedRows(),
            }).then(() => {
                Utilities.showSuccessMessage();
                component.datasource.get();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                component.cancelEditedRows();
                LoadingService.hide();
            });
        }
    }

    public messageConfirmDelete({ event }: any) {
        Utilities.showConfirmDeleteMessage(event, 'FmtAtivosInsumosCategoriasController.deleteCategorias', 'Excluir Categorias', 'Deseja realmente excluir os registros selecionados?');
    }

    public deleteCategorias() {
        const grid = Metadata.getInstance<TekGrid>('tek-grid-categorias');

        LoadingService.show();
        Http.post('/deleteCategorias', {
            ROWS: grid.selectedRows.map((el: any) => el.NRCATEGORIAEQUIP),
        }).then(() => {
            Utilities.showSuccessMessage();
            grid.datasource.get();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            grid.selectedRows = [];
            DialogService.hide();
            LoadingService.hide();
        });
    }

    public setFilterGrid({ component }: IEventParam<TekTreeGrid>) {
        component.datasource.setFilter({
            NRCATEGORIAEQUIP: this.currentRow.NRCATEGORIAEQUIP,
        });
    }

    public rowClick({ row, component }: any) {
        if (component.editing === false) {
            this.currentRow = row;
            this.abrirModalVisualizacao();
        }
    }

    public cancelarEdicao({ component }: any) {
        setTimeout(() => {
            component.cancelEditedRows();
        }, 4300);
    }
}
